function joinArray$1(arr) {
    let i, idx = 0;
    const size = arr.reduce((prev, curr) => prev + curr.length, 0);
    const buff = new Uint8Array(size);
    for (const bytes of arr) {
        for (i = 0; i < bytes.length; idx++, i++) {
            buff[idx] = bytes[i];
        }
    }
    return buff;
}

const digestLength$1 = 32;
const blockSize$1 = 64;
const K$1 = new Uint32Array([
    0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5, 0x3956c25b,
    0x59f111f1, 0x923f82a4, 0xab1c5ed5, 0xd807aa98, 0x12835b01,
    0x243185be, 0x550c7dc3, 0x72be5d74, 0x80deb1fe, 0x9bdc06a7,
    0xc19bf174, 0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc,
    0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da, 0x983e5152,
    0xa831c66d, 0xb00327c8, 0xbf597fc7, 0xc6e00bf3, 0xd5a79147,
    0x06ca6351, 0x14292967, 0x27b70a85, 0x2e1b2138, 0x4d2c6dfc,
    0x53380d13, 0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
    0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3, 0xd192e819,
    0xd6990624, 0xf40e3585, 0x106aa070, 0x19a4c116, 0x1e376c08,
    0x2748774c, 0x34b0bcb5, 0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f,
    0x682e6ff3, 0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208,
    0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2
]);
function hashBlocks$1(w, v, p, pos, len) {
    let a, b, c, d, e, f, g, h, u, i, j, t1, t2;
    while (len >= 64) {
        a = v[0];
        b = v[1];
        c = v[2];
        d = v[3];
        e = v[4];
        f = v[5];
        g = v[6];
        h = v[7];
        for (i = 0; i < 16; i++) {
            j = pos + i * 4;
            w[i] = (((p[j] & 0xff) << 24) | ((p[j + 1] & 0xff) << 16) |
                ((p[j + 2] & 0xff) << 8) | (p[j + 3] & 0xff));
        }
        for (i = 16; i < 64; i++) {
            u = w[i - 2];
            t1 = (u >>> 17 | u << (32 - 17)) ^ (u >>> 19 | u << (32 - 19)) ^ (u >>> 10);
            u = w[i - 15];
            t2 = (u >>> 7 | u << (32 - 7)) ^ (u >>> 18 | u << (32 - 18)) ^ (u >>> 3);
            w[i] = (t1 + w[i - 7] | 0) + (t2 + w[i - 16] | 0);
        }
        for (i = 0; i < 64; i++) {
            t1 = (((((e >>> 6 | e << (32 - 6)) ^ (e >>> 11 | e << (32 - 11)) ^
                (e >>> 25 | e << (32 - 25))) + ((e & f) ^ (~e & g))) | 0) +
                ((h + ((K$1[i] + w[i]) | 0)) | 0)) | 0;
            t2 = (((a >>> 2 | a << (32 - 2)) ^ (a >>> 13 | a << (32 - 13)) ^
                (a >>> 22 | a << (32 - 22))) + ((a & b) ^ (a & c) ^ (b & c))) | 0;
            h = g;
            g = f;
            f = e;
            e = (d + t1) | 0;
            d = c;
            c = b;
            b = a;
            a = (t1 + t2) | 0;
        }
        v[0] += a;
        v[1] += b;
        v[2] += c;
        v[3] += d;
        v[4] += e;
        v[5] += f;
        v[6] += g;
        v[7] += h;
        pos += 64;
        len -= 64;
    }
    return pos;
}
class Hash {
    constructor() {
        this.digestLength = digestLength$1;
        this.blockSize = blockSize$1;
        this.state = new Int32Array(8);
        this.temp = new Int32Array(64);
        this.buffer = new Uint8Array(128);
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        this.reset();
    }
    reset() {
        this.state[0] = 0x6a09e667;
        this.state[1] = 0xbb67ae85;
        this.state[2] = 0x3c6ef372;
        this.state[3] = 0xa54ff53a;
        this.state[4] = 0x510e527f;
        this.state[5] = 0x9b05688c;
        this.state[6] = 0x1f83d9ab;
        this.state[7] = 0x5be0cd19;
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        return this;
    }
    clean() {
        for (let i = 0; i < this.buffer.length; i++) {
            this.buffer[i] = 0;
        }
        for (let i = 0; i < this.temp.length; i++) {
            this.temp[i] = 0;
        }
        this.reset();
    }
    update(data, dataLength = data.length) {
        if (this.finished) {
            throw new Error("SHA256: can't update because hash was finished.");
        }
        let dataPos = 0;
        this.bytesHashed += dataLength;
        if (this.bufferLength > 0) {
            while (this.bufferLength < 64 && dataLength > 0) {
                this.buffer[this.bufferLength++] = data[dataPos++];
                dataLength--;
            }
            if (this.bufferLength === 64) {
                hashBlocks$1(this.temp, this.state, this.buffer, 0, 64);
                this.bufferLength = 0;
            }
        }
        if (dataLength >= 64) {
            dataPos = hashBlocks$1(this.temp, this.state, data, dataPos, dataLength);
            dataLength %= 64;
        }
        while (dataLength > 0) {
            this.buffer[this.bufferLength++] = data[dataPos++];
            dataLength--;
        }
        return this;
    }
    finish(out) {
        if (!this.finished) {
            const bytesHashed = this.bytesHashed;
            const left = this.bufferLength;
            const bitLenHi = (bytesHashed / 0x20000000) | 0;
            const bitLenLo = bytesHashed << 3;
            const padLength = (bytesHashed % 64 < 56) ? 64 : 128;
            this.buffer[left] = 0x80;
            for (let i = left + 1; i < padLength - 8; i++) {
                this.buffer[i] = 0;
            }
            this.buffer[padLength - 8] = (bitLenHi >>> 24) & 0xff;
            this.buffer[padLength - 7] = (bitLenHi >>> 16) & 0xff;
            this.buffer[padLength - 6] = (bitLenHi >>> 8) & 0xff;
            this.buffer[padLength - 5] = (bitLenHi >>> 0) & 0xff;
            this.buffer[padLength - 4] = (bitLenLo >>> 24) & 0xff;
            this.buffer[padLength - 3] = (bitLenLo >>> 16) & 0xff;
            this.buffer[padLength - 2] = (bitLenLo >>> 8) & 0xff;
            this.buffer[padLength - 1] = (bitLenLo >>> 0) & 0xff;
            hashBlocks$1(this.temp, this.state, this.buffer, 0, padLength);
            this.finished = true;
        }
        for (let i = 0; i < 8; i++) {
            out[i * 4 + 0] = (this.state[i] >>> 24) & 0xff;
            out[i * 4 + 1] = (this.state[i] >>> 16) & 0xff;
            out[i * 4 + 2] = (this.state[i] >>> 8) & 0xff;
            out[i * 4 + 3] = (this.state[i] >>> 0) & 0xff;
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
    _saveState(out) {
        for (let i = 0; i < this.state.length; i++) {
            out[i] = this.state[i];
        }
    }
    _restoreState(from, bytesHashed) {
        for (let i = 0; i < this.state.length; i++) {
            this.state[i] = from[i];
        }
        this.bytesHashed = bytesHashed;
        this.finished = false;
        this.bufferLength = 0;
    }
}
let HMAC$1 = class HMAC {
    constructor(key) {
        this.inner = new Hash();
        this.outer = new Hash();
        this.blockSize = this.inner.blockSize;
        this.digestLength = this.inner.digestLength;
        const pad = new Uint8Array(this.blockSize);
        if (key.length > this.blockSize) {
            (new Hash()).update(key).finish(pad).clean();
        }
        else {
            for (let i = 0; i < key.length; i++) {
                pad[i] = key[i];
            }
        }
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36;
        }
        this.inner.update(pad);
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36 ^ 0x5c;
        }
        this.outer.update(pad);
        this.istate = new Uint32Array(8);
        this.ostate = new Uint32Array(8);
        this.inner._saveState(this.istate);
        this.outer._saveState(this.ostate);
        for (let i = 0; i < pad.length; i++) {
            pad[i] = 0;
        }
    }
    reset() {
        this.inner._restoreState(this.istate, this.inner.blockSize);
        this.outer._restoreState(this.ostate, this.outer.blockSize);
        return this;
    }
    clean() {
        for (let i = 0; i < this.istate.length; i++) {
            this.ostate[i] = this.istate[i] = 0;
        }
        this.inner.clean();
        this.outer.clean();
    }
    update(data) {
        this.inner.update(data);
        return this;
    }
    finish(out) {
        if (this.outer.finished) {
            this.outer.finish(out);
        }
        else {
            this.inner.finish(out);
            this.outer.update(out, this.digestLength).finish(out);
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
};
function sha256(data) {
    const h = (new Hash()).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}
function hash256(data) {
    return sha256(sha256(data));
}
function hmac256(key, data) {
    const h = (new HMAC$1(key)).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}

const ec$2$1 = new TextEncoder();
const ALPHABETS$1 = [
    {
        name: 'base58',
        charset: '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
    }
];
function getAlphabet$1(name) {
    for (const alpha of ALPHABETS$1) {
        if (alpha.name === name) {
            return alpha.charset;
        }
    }
    throw TypeError('Charset does not exist: ' + name);
}
function encode$1$1(data, charset, padding = false) {
    if (typeof data === 'string')
        data = ec$2$1.encode(data);
    const alphabet = getAlphabet$1(charset);
    const len = alphabet.length;
    const d = [];
    let s = '', i, j = 0, c, n;
    for (i = 0; i < data.length; i++) {
        j = 0;
        c = data[i];
        s += (c > 0 || (s.length ^ i) > 0) ? '' : '1';
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * 256 + c : c;
            c = n / len | 0;
            d[j] = n % len;
            j++;
        }
    }
    while (j-- > 0) {
        s += alphabet[d[j]];
    }
    return (padding && s.length % 4 > 0)
        ? s + '='.repeat(4 - s.length % 4)
        : s;
}
function decode$1$1(encoded, charset) {
    const alphabet = getAlphabet$1(charset);
    const len = alphabet.length, d = [], b = [];
    encoded = encoded.replace('=', '');
    let i, j = 0, c, n;
    for (i = 0; i < encoded.length; i++) {
        j = 0;
        c = alphabet.indexOf(encoded[i]);
        if (c < 0) {
            throw new Error(`Character range out of bounds: ${c}`);
        }
        if (!(c > 0 || (b.length ^ i) > 0))
            b.push(0);
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * len + c : c;
            c = n >> 8;
            d[j] = n % 256;
            j++;
        }
    }
    while (j-- > 0) {
        b.push(d[j]);
    }
    return new Uint8Array(b);
}
function addChecksum$1(data) {
    const sum = hash256(data);
    return joinArray$1([data, sum.slice(0, 4)]);
}
function checkTheSum$1(data) {
    const ret = data.slice(0, -4);
    const chk = data.slice(-4);
    const sum = hash256(ret).slice(0, 4);
    if (sum.toString() !== chk.toString()) {
        throw new Error('Invalid checksum!');
    }
    return ret;
}
const BaseX$1 = {
    encode: encode$1$1,
    decode: decode$1$1
};
const Base58C$1 = {
    encode: (data) => {
        const withSum = addChecksum$1(data);
        return BaseX$1.encode(withSum, 'base58');
    },
    decode: (data) => {
        const decoded = BaseX$1.decode(data, 'base58');
        return checkTheSum$1(decoded);
    }
};

const CHARSET$1 = 'qpzry9x8gf2tvdw0s3jn54khce6mua7l';
const GENERATOR$1 = [0x3b6a57b2, 0x26508e6d, 0x1ea119fa, 0x3d4233dd, 0x2a1462b3];
const ENCODINGS$1 = [
    { version: 0, name: 'bech32', const: 1 },
    { version: 1, name: 'bech32m', const: 0x2bc830a3 }
];
function polymod$1(values) {
    let chk = 1;
    for (let p = 0; p < values.length; ++p) {
        const top = chk >> 25;
        chk = (chk & 0x1ffffff) << 5 ^ values[p];
        for (let i = 0; i < 5; ++i) {
            if (((top >> i) & 1) !== 0) {
                chk ^= GENERATOR$1[i];
            }
        }
    }
    return chk;
}
function hrpExpand$1(hrp) {
    const ret = [];
    let p;
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) >> 5);
    }
    ret.push(0);
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) & 31);
    }
    return ret;
}
function verifyChecksum$1(hrp, data, enc) {
    const combined = hrpExpand$1(hrp).concat(data);
    return polymod$1(combined) === enc.const;
}
function createChecksum$1(hrp, data, enc) {
    const values = hrpExpand$1(hrp).concat(data).concat([0, 0, 0, 0, 0, 0]);
    const mod = polymod$1(values) ^ enc.const;
    const ret = [];
    for (let p = 0; p < 6; ++p) {
        ret.push((mod >> 5 * (5 - p)) & 31);
    }
    return ret;
}
function convertBits$1(data, fromBits, toBits, pad = true) {
    const ret = [];
    let acc = 0;
    let bits = 0;
    const maxVal = (1 << toBits) - 1;
    const maxAcc = (1 << (fromBits + toBits - 1)) - 1;
    for (const val of data) {
        if (val < 0 || (val >> fromBits) > 0) {
            throw new Error('Failed to perform base conversion. Invalid value: ' + String(val));
        }
        acc = ((acc << fromBits) | val) & maxAcc;
        bits += fromBits;
        while (bits >= toBits) {
            bits -= toBits;
            ret.push((acc >> bits) & maxVal);
        }
    }
    if (pad) {
        if (bits > 0) {
            ret.push((acc << (toBits - bits)) & maxVal);
        }
    }
    else if (bits >= fromBits || ((acc << (toBits - bits)) & maxVal) > 0) {
        throw new Error('Failed to perform base conversion. Invalid Size!');
    }
    return ret;
}
function encode$7(hrp, data, enc) {
    const combined = data.concat(createChecksum$1(hrp, data, enc));
    let ret = hrp + '1';
    for (let p = 0; p < combined.length; ++p) {
        ret += CHARSET$1.charAt(combined[p]);
    }
    return ret;
}
function decode$7(bechstr) {
    if (!checkBounds$1(bechstr)) {
        throw new Error('Encoded string goes out of bounds!');
    }
    bechstr = bechstr.toLowerCase();
    if (!checkSeparatorPos$1(bechstr)) {
        throw new Error('Encoded string has invalid separator!');
    }
    const data = [];
    const pos = bechstr.lastIndexOf('1');
    const hrp = bechstr.substring(0, pos);
    for (let p = pos + 1; p < bechstr.length; ++p) {
        const d = CHARSET$1.indexOf(bechstr.charAt(p));
        if (d === -1) {
            throw new Error('Character idx out of bounds: ' + String(p));
        }
        data.push(d);
    }
    const enc = ENCODINGS$1.find(e => e.version === data[0]) ?? ENCODINGS$1[0];
    if (!verifyChecksum$1(hrp, data, enc)) {
        throw new Error('Checksum verification failed!');
    }
    return [hrp, data.slice(0, data.length - 6)];
}
function checkBounds$1(bechstr) {
    let p;
    let char;
    let hasLower = false;
    let hasUpper = false;
    for (p = 0; p < bechstr.length; ++p) {
        char = bechstr.charCodeAt(p);
        if (char < 33 || char > 126) {
            return false;
        }
        if (char >= 97 && char <= 122) {
            hasLower = true;
        }
        if (char >= 65 && char <= 90) {
            hasUpper = true;
        }
    }
    if (hasLower && hasUpper)
        return false;
    return true;
}
function checkSeparatorPos$1(bechstr) {
    const pos = bechstr.lastIndexOf('1');
    return !(pos < 1 ||
        pos + 7 > bechstr.length ||
        bechstr.length > 90);
}
function b32encode$1(data, hrp = 'bc', version = 0) {
    const dat = [version, ...convertBits$1([...data], 8, 5)];
    const enc = ENCODINGS$1.find(e => e.version === version) ?? ENCODINGS$1[0];
    const str = encode$7(hrp, dat, enc);
    b32decode$1(str);
    return str;
}
function b32decode$1(str) {
    str = str.toLowerCase();
    const hrp = str.split('1', 1)[0];
    const [hrpgot, data] = decode$7(str);
    const decoded = convertBits$1(data.slice(1), 5, 8, false);
    const length = decoded.length;
    switch (true) {
        case (hrp !== hrpgot):
            throw new Error('Returned hrp string is invalid.');
        case (decoded === null || length < 2 || length > 40):
            throw new Error('Decoded string is invalid or out of spec.');
        case (data[0] > 16):
            throw new Error('Returned version bit is out of range.');
        default:
            return Uint8Array.from(decoded);
    }
}
function getVersion$2(str) {
    str = str.toLowerCase();
    const [_, data] = decode$7(str);
    return data[0];
}
const Bech32$1 = {
    encode: b32encode$1,
    decode: b32decode$1,
    version: getVersion$2
};

const BASE64_MAP$1 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
const B64URL_MAP$1 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_';
const ec$1$1 = new TextEncoder();
function b64encode$1(input, urlSafe = false) {
    if (typeof input === 'string')
        input = ec$1$1.encode(input);
    const map = urlSafe ? B64URL_MAP$1 : BASE64_MAP$1;
    let output = '';
    let bits = 0;
    let buffer = 0;
    for (let i = 0; i < input.length; i++) {
        buffer = (buffer << 8) | input[i];
        bits += 8;
        while (bits >= 6) {
            bits -= 6;
            output += map[(buffer >> bits) & 0x3f];
        }
    }
    if (bits > 0) {
        buffer <<= 6 - bits;
        output += map[buffer & 0x3f];
        while (bits < 6) {
            output += urlSafe ? '' : '=';
            bits += 2;
        }
    }
    return output;
}
function b64decode$1(input, urlSafe = false) {
    const map = (urlSafe || input.includes('-') || input.includes('_'))
        ? B64URL_MAP$1.split('')
        : BASE64_MAP$1.split('');
    input = input.replace(/=+$/, '');
    const chars = input.split('');
    let bits = 0;
    let value = 0;
    const bytes = [];
    for (let i = 0; i < chars.length; i++) {
        const c = chars[i];
        const index = map.indexOf(c);
        if (index === -1) {
            throw new Error('Invalid character: ' + c);
        }
        bits += 6;
        value <<= 6;
        value |= index;
        if (bits >= 8) {
            bits -= 8;
            bytes.push((value >>> bits) & 0xff);
        }
    }
    return new Uint8Array(bytes);
}
const Base64$1 = {
    encode: b64encode$1,
    decode: b64decode$1
};
const B64URL$1 = {
    encode: (data) => b64encode$1(data, true),
    decode: (data) => b64decode$1(data, true)
};

const ec$3 = new TextEncoder();
const dc$1 = new TextDecoder();
function strToBytes$1(str) {
    return ec$3.encode(str);
}
function hexToBytes$2(str) {
    const bytes = new Uint8Array(str.length / 2);
    let i, idx = 0;
    if (str.match(/[^a-fA-f0-9]/) !== null) {
        throw new TypeError('Invalid hex string: ' + str);
    }
    if (str.length % 2 > 0) {
        throw new Error(`Hex string length is uneven: ${str.length}`);
    }
    for (i = 0; i < str.length; i += 2) {
        bytes[idx] = parseInt(str.slice(i, i + 2), 16);
        idx += 1;
    }
    return bytes;
}
function numToBytes$1(num) {
    if (num === 0)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xff;
        bytes.push(byte);
        num = (num - byte) / 256;
    }
    return new Uint8Array(bytes);
}
function binaryToBytes$1(binary) {
    if (typeof binary === 'string') {
        binary = binary.split('').map(Number);
    }
    else if (!Array.isArray(binary)) {
        throw new Error('Invalid input type: expected a string or an array of numbers.');
    }
    if (binary.length % 8 !== 0) {
        throw new Error(`Binary array is invalid length: ${binary.length}`);
    }
    const bytes = new Uint8Array(binary.length / 8);
    for (let i = 0, ct = 0; i < binary.length; i += 8, ct++) {
        let byte = 0;
        for (let j = 0; j < 8; j++) {
            byte |= (binary[i + j] << (7 - j));
        }
        bytes[ct] = byte;
    }
    return bytes;
}
function bytesToBinary$1(bytes) {
    const bin = new Array(bytes.length * 8);
    let count = 0;
    for (const num of bytes) {
        if (num > 255) {
            throw new Error(`Invalid byte value: ${num}. Byte values must be between 0 and 255.`);
        }
        for (let i = 7; i >= 0; i--, count++) {
            bin[count] = (num >> i) & 1;
        }
    }
    return bin;
}
function bigToBytes$1(big) {
    if (big === 0n)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (big > 0n) {
        const byte = big & 0xffn;
        bytes.push(Number(byte));
        big = (big - byte) / 256n;
    }
    return new Uint8Array(bytes);
}
function bytesToStr$1(bytes) {
    return dc$1.decode(bytes);
}
function bytesToHex$2(bytes) {
    const chars = new Array(bytes.length);
    for (let i = 0; i < bytes.length; i++) {
        chars.push(bytes[i].toString(16).padStart(2, '0'));
    }
    return chars.join('');
}
function bytesToNum$1(bytes) {
    let num = 0, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256) + bytes[i];
    }
    return Number(num);
}
function bytesToBig$1(bytes) {
    let num = 0n, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256n) + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function buffer$1(value, bytes = true) {
    if (value instanceof ArrayBuffer) {
        return new Uint8Array(value);
    }
    if (value instanceof Uint8Array) {
        return value;
    }
    switch (typeof value) {
        case 'bigint':
            return bigToBytes$1(value);
        case 'boolean':
            return Uint8Array.of(value ? 1 : 0);
        case 'number':
            return numToBytes$1(value);
        case 'string':
            return (bytes)
                ? hexToBytes$2(value)
                : ec$3.encode(value);
        default:
            throw TypeError('Unsupported format:' + String(typeof value));
    }
}
function hexify$1(bytes) {
    bytes = buffer$1(bytes, true);
    return bytesToHex$2(bytes);
}
function serialize$1(data) {
    if (typeof data === 'object') {
        if (data instanceof Uint8Array) {
            return data;
        }
        try {
            return strToBytes$1(JSON.stringify(data));
        }
        catch {
            throw TypeError('Object is not serializable.');
        }
    }
    return buffer$1(data, false);
}
function revive$1(data) {
    if (data instanceof Uint8Array) {
        data = bytesToStr$1(data);
    }
    if (typeof data === 'string') {
        try {
            return JSON.parse(data);
        }
        catch {
            return data;
        }
    }
    return data;
}

function bytesToBigInt(bytes) {
    let num = 0n;
    for (let i = bytes.length - 1; i >= 0; i--) {
        num = num * 256n + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function bigIntToBytes(num, size = 0) {
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xffn;
        bytes.push(byte);
        num = (num - byte) / 256n;
    }
    let uint8 = Uint8Array.from(bytes.map((n) => Number(n)));
    if (size !== 0) {
        const resized = new Uint8Array(size);
        resized.set(uint8);
        uint8 = resized;
    }
    return uint8;
}
const ML = [
    0n,
    1n,
    2n,
    3n,
    4n,
    5n,
    6n,
    7n,
    8n,
    9n,
    10n,
    11n,
    12n,
    13n,
    14n,
    15n,
    7n,
    4n,
    13n,
    1n,
    10n,
    6n,
    15n,
    3n,
    12n,
    0n,
    9n,
    5n,
    2n,
    14n,
    11n,
    8n,
    3n,
    10n,
    14n,
    4n,
    9n,
    15n,
    8n,
    1n,
    2n,
    7n,
    0n,
    6n,
    13n,
    11n,
    5n,
    12n,
    1n,
    9n,
    11n,
    10n,
    0n,
    8n,
    12n,
    4n,
    13n,
    3n,
    7n,
    15n,
    14n,
    5n,
    6n,
    2n,
    4n,
    0n,
    5n,
    9n,
    7n,
    12n,
    2n,
    10n,
    14n,
    1n,
    3n,
    8n,
    11n,
    6n,
    15n,
    13n
];
const MR = [
    5n,
    14n,
    7n,
    0n,
    9n,
    2n,
    11n,
    4n,
    13n,
    6n,
    15n,
    8n,
    1n,
    10n,
    3n,
    12n,
    6n,
    11n,
    3n,
    7n,
    0n,
    13n,
    5n,
    10n,
    14n,
    15n,
    8n,
    12n,
    4n,
    9n,
    1n,
    2n,
    15n,
    5n,
    1n,
    3n,
    7n,
    14n,
    6n,
    9n,
    11n,
    8n,
    12n,
    2n,
    10n,
    0n,
    4n,
    13n,
    8n,
    6n,
    4n,
    1n,
    3n,
    11n,
    15n,
    0n,
    5n,
    12n,
    2n,
    13n,
    9n,
    7n,
    10n,
    14n,
    12n,
    15n,
    10n,
    4n,
    1n,
    5n,
    8n,
    7n,
    6n,
    2n,
    13n,
    14n,
    0n,
    3n,
    9n,
    11n
];
const RL = [
    11n,
    14n,
    15n,
    12n,
    5n,
    8n,
    7n,
    9n,
    11n,
    13n,
    14n,
    15n,
    6n,
    7n,
    9n,
    8n,
    7n,
    6n,
    8n,
    13n,
    11n,
    9n,
    7n,
    15n,
    7n,
    12n,
    15n,
    9n,
    11n,
    7n,
    13n,
    12n,
    11n,
    13n,
    6n,
    7n,
    14n,
    9n,
    13n,
    15n,
    14n,
    8n,
    13n,
    6n,
    5n,
    12n,
    7n,
    5n,
    11n,
    12n,
    14n,
    15n,
    14n,
    15n,
    9n,
    8n,
    9n,
    14n,
    5n,
    6n,
    8n,
    6n,
    5n,
    12n,
    9n,
    15n,
    5n,
    11n,
    6n,
    8n,
    13n,
    12n,
    5n,
    12n,
    13n,
    14n,
    11n,
    8n,
    5n,
    6n
];
const RR = [
    8n,
    9n,
    9n,
    11n,
    13n,
    15n,
    15n,
    5n,
    7n,
    7n,
    8n,
    11n,
    14n,
    14n,
    12n,
    6n,
    9n,
    13n,
    15n,
    7n,
    12n,
    8n,
    9n,
    11n,
    7n,
    7n,
    12n,
    7n,
    6n,
    15n,
    13n,
    11n,
    9n,
    7n,
    15n,
    11n,
    8n,
    6n,
    6n,
    14n,
    12n,
    13n,
    5n,
    14n,
    13n,
    13n,
    7n,
    5n,
    15n,
    5n,
    8n,
    11n,
    14n,
    14n,
    6n,
    14n,
    6n,
    9n,
    12n,
    9n,
    12n,
    5n,
    15n,
    8n,
    8n,
    5n,
    12n,
    9n,
    12n,
    5n,
    14n,
    6n,
    8n,
    13n,
    6n,
    5n,
    15n,
    13n,
    11n,
    11n
];
const KL = [0n, 0x5a827999n, 0x6ed9eba1n, 0x8f1bbcdcn, 0xa953fd4en];
const KR = [0x50a28be6n, 0x5c4dd124n, 0x6d703ef3n, 0x7a6d76e9n, 0n];
function fi(x, y, z, i) {
    switch (true) {
        case i === 0n:
            return x ^ y ^ z;
        case i === 1n:
            return (x & y) | (~x & z);
        case i === 2n:
            return (x | ~y) ^ z;
        case i === 3n:
            return (x & z) | (y & ~z);
        case i === 4n:
            return x ^ (y | ~z);
        default:
            throw new TypeError('Unknown I value: ' + String(i));
    }
}
function rol(x, i) {
    return ((x << i) | ((x & 0xffffffffn) >> (32n - i))) & 0xffffffffn;
}
function compress(h0, h1, h2, h3, h4, block) {
    const x = [];
    let rnd, elt, ert;
    let ml, kl, mr, kr;
    let al = h0, bl = h1, cl = h2, dl = h3, el = h4;
    let ar = h0, br = h1, cr = h2, dr = h3, er = h4;
    for (let i = 0; i < 16; i++) {
        const num = bytesToBigInt(block.slice(4 * i, 4 * (i + 1)));
        x.push(num);
    }
    for (let i = 0; i < 80; i++) {
        rnd = BigInt(i) >> 4n;
        ml = x[Number(ML[i])];
        kl = KL[Number(rnd)];
        mr = x[Number(MR[i])];
        kr = KR[Number(rnd)];
        al = rol(al + fi(bl, cl, dl, rnd) + ml + kl, RL[i]) + el;
        elt = el;
        el = dl;
        dl = rol(cl, 10n);
        cl = bl;
        bl = al;
        al = elt;
        ar = rol(ar + fi(br, cr, dr, 4n - rnd) + mr + kr, RR[i]) + er;
        ert = er;
        er = dr;
        dr = rol(cr, 10n);
        cr = br;
        br = ar;
        ar = ert;
    }
    return [h1 + cl + dr, h2 + dl + er, h3 + el + ar, h4 + al + br, h0 + bl + cr];
}
function ripemd160(bytes) {
    let state = [
        0x67452301n,
        0xefcdab89n,
        0x98badcfen,
        0x10325476n,
        0xc3d2e1f0n
    ];
    for (let b = 0; b < bytes.length >> 6; b++) {
        state = compress(...state, bytes.slice(64 * b, 64 * (b + 1)));
    }
    const zfill = new Array((119 - bytes.length) & 63).fill(0);
    const pad = [0x80, ...zfill];
    const fin = Uint8Array.from([
        ...bytes.slice(bytes.length & ~63),
        ...pad,
        ...bigIntToBytes(BigInt(8 * bytes.length), 8)
    ]);
    for (let i = 0; i < fin.length >> 6; i++) {
        state = compress(...state, fin.slice(64 * i, 64 * (i + 1)));
    }
    const ret = [];
    for (let i = 0; i < state.length; i++) {
        const num = state[i] & 0xffffffffn;
        ret.push(...bigIntToBytes(num, 4));
    }
    return Uint8Array.from(ret);
}

let PRNG$1 = class PRNG {
    static seed(size = 32) {
        const date = numToBytes$1(Date.now());
        const seed = new Uint8Array(size * 4).fill(0);
        seed.map(() => Math.floor(Math.random() * 0xFFFF % 0xFF));
        return sha256(joinArray$1([seed, date]));
    }
    static getRandomValues(array) {
        if (array !== null) {
            const view = new Uint8Array(array.buffer);
            const size = view.length;
            const rand = PRNG$1.generate(size);
            for (let i = 0; i < size; i++) {
                view[i] = rand[i];
            }
            return array;
        }
        throw new Error('ArrayBufferView is null!');
    }
    static generate(size = 32) {
        let seed = PRNG$1.seed();
        const rnds = Math.ceil((size + 32) / 32);
        const pool = [];
        for (let i = 0; i < rnds; i++) {
            const prng = new PRNG$1(seed);
            const hash = prng.generate();
            seed = hash;
            pool.push(hash);
        }
        return joinArray$1(pool.reverse()).slice(0, size);
    }
    constructor(seed = PRNG$1.seed()) {
        this.x = bytesToBig$1(seed);
        this.y = BigInt(362436069);
        this.z = BigInt(521288629);
        this.w = BigInt(88675123);
        this.c = BigInt(0);
    }
    generate() {
        const next = this.next();
        if (next < (2n ** 512n)) {
            return this.generate();
        }
        return sha256(bigToBytes$1(next));
    }
    next() {
        this.x = BigInt(6906969069) * this.x + this.c;
        this.y ^= this.y << BigInt(13);
        this.y ^= this.y >> BigInt(17);
        this.y ^= this.y << BigInt(43);
        const t = this.z + this.w + this.c;
        this.z = this.w;
        this.c = t >> BigInt(63);
        this.w = t & BigInt('0x7fffffffffffffff');
        return this.x + this.y + this.w;
    }
};

const webcrypto$1 = (typeof crypto !== 'undefined')
    ? crypto
    : (typeof globalThis?.crypto !== 'undefined')
        ? globalThis.crypto
        : (typeof window !== 'undefined')
            ? window.crypto
            : {
                getRandomValues: PRNG$1.getRandomValues,
                subtle: undefined
            };

let Buff$1 = class Buff extends Uint8Array {
    static { this.num = (number, size, endian = 'le') => {
        const b = new Buff$1(numToBytes$1(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static { this.big = (number, size, endian = 'le') => {
        const b = new Buff$1(bigToBytes$1(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static { this.bin = (data, size) => new Buff$1(binaryToBytes$1(data), size); }
    static { this.any = (data, size) => new Buff$1(buffer$1(data, false), size); }
    static { this.raw = (data, size) => new Buff$1(data, size); }
    static { this.str = (data, size) => new Buff$1(strToBytes$1(data), size); }
    static { this.hex = (data, size) => new Buff$1(hexToBytes$2(data), size); }
    static { this.json = (data) => new Buff$1(strToBytes$1(JSON.stringify(data))); }
    static { this.bytes = (data, size) => new Buff$1(buffer$1(data, true), size); }
    static { this.base64 = (data) => new Buff$1(Base64$1.decode(data)); }
    static { this.b64url = (data) => new Buff$1(B64URL$1.decode(data)); }
    static { this.bech32 = (data) => new Buff$1(Bech32$1.decode(data)); }
    static { this.b58chk = (data) => new Buff$1(Base58C$1.decode(data)); }
    constructor(data, size) {
        data = buffer$1(data, true);
        if (typeof size === 'number') {
            const tmp = new Uint8Array(size).fill(0);
            tmp.set(new Uint8Array(data));
            data = tmp.buffer;
        }
        super(data);
        return this;
    }
    get arr() {
        return [...this];
    }
    get num() {
        return this.toNum();
    }
    get big() {
        return this.toBig();
    }
    get str() {
        return this.toStr();
    }
    get hex() {
        return this.toHex();
    }
    get raw() {
        return new Uint8Array(this);
    }
    get bits() {
        return this.toBits();
    }
    get bin() {
        return this.toBin();
    }
    get b58chk() {
        return this.tob58chk();
    }
    get base64() {
        return this.toBase64();
    }
    get b64url() {
        return this.toB64url();
    }
    get digest() {
        return this.toHash();
    }
    get id() {
        return this.toHash().hex;
    }
    get stream() {
        return new Stream$1(this);
    }
    toNum(endian = 'le') {
        return (endian === 'le')
            ? bytesToNum$1(this.reverse())
            : bytesToNum$1(this);
    }
    toBig(endian = 'le') {
        return (endian === 'le')
            ? bytesToBig$1(this.reverse())
            : bytesToBig$1(this);
    }
    toHash(type = 'sha256') {
        switch (type) {
            case 'sha256':
                return new Buff$1(sha256(this));
            case 'hash256':
                return new Buff$1(sha256(sha256(this)));
            case 'ripe160':
                return new Buff$1(ripemd160(this));
            case 'hash160':
                return new Buff$1(ripemd160(sha256(this)));
            default:
                throw new Error('Unrecognized format:' + String(type));
        }
    }
    toHmac(key) {
        if (typeof key === 'string') {
            key = strToBytes$1(key);
        }
        return new Buff$1(hmac256(key, this));
    }
    toStr() { return bytesToStr$1(this); }
    toHex() { return bytesToHex$2(this); }
    toJson() { return JSON.parse(bytesToStr$1(this)); }
    toBytes() { return new Uint8Array(this); }
    toBits() { return bytesToBinary$1(this); }
    toBin() { return bytesToBinary$1(this).join(''); }
    tob58chk() { return Base58C$1.encode(this); }
    toB64url() { return B64URL$1.encode(this); }
    toBase64() { return Base64$1.encode(this); }
    toBech32(hrp, version = 0) { return Bech32$1.encode(this, hrp, version); }
    prepend(data) {
        return Buff$1.join([Buff$1.bytes(data), this]);
    }
    append(data) {
        return Buff$1.join([this, Buff$1.bytes(data)]);
    }
    slice(start, end) {
        return new Buff$1(new Uint8Array(this).slice(start, end));
    }
    reverse() {
        return new Buff$1(new Uint8Array(this).reverse());
    }
    write(bytes, offset) {
        this.set(bytes, offset);
    }
    prefixSize(endian) {
        const size = Buff$1.varInt(this.length, endian);
        return Buff$1.join([size, this]);
    }
    static from(data) {
        return new Buff$1(Uint8Array.from(data));
    }
    static of(...args) {
        return new Buff$1(Uint8Array.of(...args));
    }
    static join(arr) {
        const data = arr.map(e => Buff$1.bytes(e));
        return new Buff$1(joinArray$1(data));
    }
    static varInt(num, endian) {
        if (num < 0xFD) {
            return Buff$1.num(num, 1);
        }
        else if (num < 0x10000) {
            return Buff$1.of(0xFD, ...Buff$1.num(num, 2, endian));
        }
        else if (num < 0x100000000) {
            return Buff$1.of(0xFE, ...Buff$1.num(num, 4, endian));
        }
        else if (BigInt(num) < 0x10000000000000000n) {
            return Buff$1.of(0xFF, ...Buff$1.num(num, 8, endian));
        }
        else {
            throw new Error(`Value is too large: ${num}`);
        }
    }
    static { this.encode = strToBytes$1; }
    static { this.decode = bytesToStr$1; }
    static random(size = 32) {
        return new Buff$1(webcrypto$1.getRandomValues(new Uint8Array(size)));
    }
    static normalize(bytes, size) {
        return new Buff$1(buffer$1(bytes, true), size);
    }
    static hexify(bytes) {
        return hexify$1(bytes);
    }
    static serialize(data, size) {
        return new Buff$1(serialize$1(data), size);
    }
    static revive(data) {
        return revive$1(data);
    }
};
let Stream$1 = class Stream {
    constructor(data) {
        this.data = new Uint8Array(data);
        this.size = this.data.length;
    }
    peek(size) {
        if (size > this.size) {
            throw new Error(`Size greater than stream: ${size} > ${this.size}`);
        }
        return new Buff$1(this.data.slice(0, size).buffer);
    }
    read(size) {
        size = size ?? this.readSize();
        const chunk = this.peek(size);
        this.data = this.data.slice(size);
        this.size = this.data.length;
        return chunk;
    }
    readSize(endian) {
        const num = this.read(1).num;
        switch (true) {
            case (num >= 0 && num < 0xFD):
                return num;
            case (num === 0xFD):
                return this.read(2).toNum(endian);
            case (num === 0xFE):
                return this.read(4).toNum(endian);
            case (num === 0xFF):
                return this.read(8).toNum(endian);
            default:
                throw new Error(`Varint is out of range: ${num}`);
        }
    }
};

function checkSize(input, size) {
    const bytes = Buff$1.bytes(input);
    if (bytes.length !== size) {
        throw new Error(`Invalid input size: ${bytes.hex} !== ${size}`);
    }
}
function safeThrow(errorMsg, shouldThrow) {
    if (shouldThrow) {
        throw new Error(errorMsg);
    }
    else {
        return false;
    }
}
function hashTag(tag, ...data) {
    const htag = Buff$1.str(tag).digest.raw;
    const buff = data.map(e => Buff$1.normalize(e));
    return Buff$1.join([htag, htag, Buff$1.join(buff)]).digest;
}

function check$4(address, network = 'main') {
    const prefixes = (network === 'main') ? ['1'] : ['m', 'n'];
    for (const prefix of prefixes) {
        if (address.startsWith(prefix)) {
            return true;
        }
    }
    return false;
}
function encode$6(input, network = 'main') {
    const bytes = Buff$1.bytes(input);
    const prefix = (network === 'main') ? Buff$1.num(0x00) : Buff$1.num(0x6F);
    checkSize(input, 20);
    return bytes.prepend(prefix).tob58chk();
}
function decode$6(address, network = 'main') {
    if (!check$4(address, network)) {
        throw new TypeError('Invalid p2pkh address!');
    }
    return Buff$1.b58chk(address).slice(1);
}
function scriptPubKey$4(input) {
    const bytes = Buff$1.bytes(input);
    checkSize(bytes, 20);
    return ['OP_DUP', 'OP_HASH160', bytes.hex, 'OP_EQUALVERIFY', 'OP_CHECKSIG'];
}
function fromPubKey$2(pubkey, network) {
    const bytes = Buff$1.bytes(pubkey);
    checkSize(bytes, 33);
    const hash = bytes.toHash('hash160');
    return encode$6(hash, network);
}
const P2PKH = { check: check$4, encode: encode$6, decode: decode$6, scriptPubKey: scriptPubKey$4, fromPubKey: fromPubKey$2 };

const OPCODE_MAP = {
    OP_0: 0,
    OP_PUSHDATA1: 76,
    OP_PUSHDATA2: 77,
    OP_PUSHDATA4: 78,
    OP_1NEGATE: 79,
    OP_SUCCESS80: 80,
    OP_1: 81,
    OP_2: 82,
    OP_3: 83,
    OP_4: 84,
    OP_5: 85,
    OP_6: 86,
    OP_7: 87,
    OP_8: 88,
    OP_9: 89,
    OP_10: 90,
    OP_11: 91,
    OP_12: 92,
    OP_13: 93,
    OP_14: 94,
    OP_15: 95,
    OP_16: 96,
    OP_NOP: 97,
    OP_SUCCESS98: 98,
    OP_IF: 99,
    OP_NOTIF: 100,
    OP_ELSE: 103,
    OP_ENDIF: 104,
    OP_VERIFY: 105,
    OP_RETURN: 106,
    OP_TOALTSTACK: 107,
    OP_FROMALTSTACK: 108,
    OP_2DROP: 109,
    OP_2DUP: 110,
    OP_3DUP: 111,
    OP_2OVER: 112,
    OP_2ROT: 113,
    OP_2SWAP: 114,
    OP_IFDUP: 115,
    OP_DEPTH: 116,
    OP_DROP: 117,
    OP_DUP: 118,
    OP_NIP: 119,
    OP_OVER: 120,
    OP_PICK: 121,
    OP_ROLL: 122,
    OP_ROT: 123,
    OP_SWAP: 124,
    OP_TUCK: 125,
    OP_SUCCESS126: 126,
    OP_SUCCESS127: 127,
    OP_SUCCESS128: 128,
    OP_SUCCESS129: 129,
    OP_SIZE: 130,
    OP_SUCCESS131: 131,
    OP_SUCCESS132: 132,
    OP_SUCCESS133: 133,
    OP_SUCCESS134: 134,
    OP_EQUAL: 135,
    OP_EQUALVERIFY: 136,
    OP_SUCCESS137: 137,
    OP_SUCCESS138: 138,
    OP_1ADD: 139,
    OP_1SUB: 140,
    OP_SUCCESS141: 141,
    OP_SUCCESS142: 142,
    OP_NEGATE: 143,
    OP_ABS: 144,
    OP_NOT: 145,
    OP_0NOTEQUAL: 146,
    OP_ADD: 147,
    OP_SUB: 148,
    OP_SUCCESS149: 149,
    OP_SUCCESS150: 150,
    OP_SUCCESS151: 151,
    OP_SUCCESS152: 152,
    OP_SUCCESS153: 153,
    OP_BOOLAND: 154,
    OP_BOOLOR: 155,
    OP_NUMEQUAL: 156,
    OP_NUMEQUALVERIFY: 157,
    OP_NUMNOTEQUAL: 158,
    OP_LESSTHAN: 159,
    OP_GREATERTHAN: 160,
    OP_LESSTHANOREQUAL: 161,
    OP_GREATERTHANOREQUAL: 162,
    OP_MIN: 163,
    OP_MAX: 164,
    OP_WITHIN: 165,
    OP_RIPEMD160: 166,
    OP_SHA1: 167,
    OP_SHA256: 168,
    OP_HASH160: 169,
    OP_HASH256: 170,
    OP_CODESEPARATOR: 171,
    OP_CHECKSIG: 172,
    OP_CHECKSIGVERIFY: 173,
    OP_CHECKMULTISIG: 174,
    OP_CHECKMULTISIGVERIFY: 175,
    OP_NOP1: 176,
    OP_CHECKLOCKTIMEVERIFY: 177,
    OP_CHECKSEQUENCEVERIFY: 178,
    OP_NOP4: 179,
    OP_NOP5: 180,
    OP_NOP6: 181,
    OP_NOP7: 182,
    OP_NOP8: 183,
    OP_NOP9: 184,
    OP_NOP10: 185,
    OP_CHECKSIGADD: 186
};
function getOpLabel(num) {
    if (num > 186 && num < 255) {
        return 'OP_SUCCESS' + String(num);
    }
    for (const [k, v] of Object.entries(OPCODE_MAP)) {
        if (v === num)
            return k;
    }
    throw new Error('OPCODE not found:' + String(num));
}
function getOpCode(string) {
    for (const [k, v] of Object.entries(OPCODE_MAP)) {
        if (k === string)
            return Number(v);
    }
    throw new Error('OPCODE not found:' + string);
}
function getWordType(word) {
    switch (true) {
        case (word === 0):
            return 'opcode';
        case (word >= 1 && word <= 75):
            return 'varint';
        case (word === 76):
            return 'pushdata1';
        case (word === 77):
            return 'pushdata2';
        case (word === 78):
            return 'pushdata4';
        case (word <= 254):
            return 'opcode';
        default:
            throw new Error(`Invalid word range: ${word}`);
    }
}
function isValidWord(word) {
    const MIN_RANGE = 75;
    const MAX_RANGE = 254;
    const DISABLED_OPCODES = [];
    switch (true) {
        case (typeof (word) !== 'number'):
            return false;
        case (word === 0):
            return true;
        case (DISABLED_OPCODES.includes(word)):
            return false;
        case (MIN_RANGE < word && word < MAX_RANGE):
            return true;
        default:
            return false;
    }
}

function isHex(value) {
    return (typeof value === 'string' &&
        value.length % 2 === 0 &&
        /[0-9a-fA-F]/.test(value));
}
function isBytes(value) {
    return (isHex(value) || value instanceof Uint8Array);
}

const MAX_WORD_SIZE = 0x208;
function encodeScript(script = [], varint = true) {
    let buff = Buff$1.num(0);
    if (Array.isArray(script)) {
        buff = Buff$1.raw(encodeWords(script));
    }
    if (isHex(script)) {
        buff = Buff$1.hex(script);
    }
    if (script instanceof Uint8Array) {
        buff = Buff$1.raw(script);
    }
    if (varint) {
        buff = buff.prefixSize('be');
    }
    return buff;
}
function encodeWords(wordArray) {
    const words = [];
    for (const word of wordArray) {
        words.push(encodeWord(word));
    }
    return (words.length > 0)
        ? Buff$1.join(words)
        : new Uint8Array();
}
function encodeWord(word) {
    let buff = new Uint8Array();
    if (typeof (word) === 'string') {
        if (word.startsWith('OP_')) {
            return Buff$1.num(getOpCode(word), 1);
        }
        else if (isHex(word)) {
            buff = Buff$1.hex(word);
        }
        else {
            buff = Buff$1.str(word);
        }
    }
    else {
        buff = Buff$1.bytes(word);
        if (buff.length === 1 && buff[0] <= 16) {
            if (buff[0] !== 0)
                buff[0] += 0x50;
            return buff;
        }
    }
    if (buff.length > MAX_WORD_SIZE) {
        const words = splitWord(buff);
        return encodeWords(words);
    }
    return Buff$1.join([encodeSize(buff.length), buff]);
}
function encodeSize(size) {
    const OP_DATAPUSH1 = Buff$1.num(0x4c, 1);
    const OP_DATAPUSH2 = Buff$1.num(0x4d, 1);
    switch (true) {
        case (size <= 0x4b):
            return Buff$1.num(size);
        case (size > 0x4b && size < 0x100):
            return Buff$1.join([OP_DATAPUSH1, Buff$1.num(size, 1)]);
        case (size >= 0x100 && size <= MAX_WORD_SIZE):
            return Buff$1.join([OP_DATAPUSH2, Buff$1.num(size, 2, 'be')]);
        default:
            throw new Error('Invalid word size:' + size.toString());
    }
}
function splitWord(word) {
    const words = [];
    const buff = new Stream$1(word);
    while (buff.size > MAX_WORD_SIZE) {
        words.push(buff.read(MAX_WORD_SIZE));
    }
    words.push(buff.read(buff.size));
    return words;
}

function decodeScript(script, varint = false) {
    let buff = Buff$1.bytes(script);
    if (varint) {
        const stream = buff.stream;
        const len = stream.readSize('be');
        if (buff.length !== len) {
            throw new Error(`Varint does not match stream size: ${len} !== ${buff.length}`);
        }
        buff = buff.slice(1);
    }
    return decodeWords(buff);
}
function decodeWords(words) {
    const stream = new Stream$1(words);
    const stack = [];
    const stackSize = stream.size;
    let word;
    let wordType;
    let wordSize;
    let count = 0;
    while (count < stackSize) {
        word = stream.read(1).num;
        wordType = getWordType(word);
        count++;
        switch (wordType) {
            case 'varint':
                stack.push(stream.read(word).hex);
                count += word;
                break;
            case 'pushdata1':
                wordSize = stream.read(1).reverse().num;
                stack.push(stream.read(wordSize).hex);
                count += wordSize + 1;
                break;
            case 'pushdata2':
                wordSize = stream.read(2).reverse().num;
                stack.push(stream.read(wordSize).hex);
                count += wordSize + 2;
                break;
            case 'pushdata4':
                wordSize = stream.read(4).reverse().num;
                stack.push(stream.read(wordSize).hex);
                count += wordSize + 4;
                break;
            case 'opcode':
                if (!isValidWord(word)) {
                    throw new Error(`Invalid OPCODE: ${word}`);
                }
                stack.push(getOpLabel(word));
                break;
            default:
                throw new Error(`Word type undefined: ${word}`);
        }
    }
    return stack;
}

function toAsm(script, varint) {
    if (Array.isArray(script)) {
        script = encodeScript(script, varint);
    }
    if (script instanceof Uint8Array ||
        isHex(script)) {
        return decodeScript(script, varint);
    }
    throw new Error('Invalid format: ' + String(typeof script));
}
function toBytes$1(script, varint) {
    if (script instanceof Uint8Array ||
        isHex(script)) {
        script = decodeScript(script, varint);
    }
    if (Array.isArray(script)) {
        return encodeScript(script, varint);
    }
    throw new Error('Invalid format: ' + String(typeof script));
}
function toParam(script) {
    if (!Array.isArray(script)) {
        return Buff$1.bytes(script);
    }
    throw new Error('Invalid format: ' + String(typeof script));
}
const FmtScript = {
    toAsm,
    toBytes: toBytes$1,
    toParam
};

const Script = {
    encode: encodeScript,
    decode: decodeScript,
    fmt: FmtScript
};

function check$3(address, network = 'main') {
    const prefixes = (network === 'main') ? ['3'] : ['2'];
    for (const prefix of prefixes) {
        if (address.startsWith(prefix)) {
            return true;
        }
    }
    return false;
}
function encode$5(input, network = 'main') {
    const prefix = (network === 'main') ? Buff$1.num(0x05) : Buff$1.num(0xC4);
    const bytes = Buff$1.bytes(input);
    checkSize(bytes, 20);
    return bytes.prepend(prefix).tob58chk();
}
function decode$5(address, network = 'main') {
    if (!check$3(address, network)) {
        throw new TypeError(`Invalid p2sh address for network ${network}:` + address);
    }
    return Buff$1.b58chk(address).slice(1);
}
function scriptPubKey$3(input) {
    const bytes = Buff$1.bytes(input);
    return ['OP_HASH160', bytes.hex, 'OP_EQUAL'];
}
function fromScript$1(script, network) {
    const bytes = Script.fmt.toBytes(script, false);
    return encode$5(bytes.toHash('hash160'), network);
}
const P2SH = { check: check$3, encode: encode$5, decode: decode$5, scriptPubKey: scriptPubKey$3, fromScript: fromScript$1 };

const BECH32_PREFIXES = {
    main: 'bc',
    testnet: 'tb',
    signet: 'tb',
    regtest: 'bcrt'
};

const VALID_PREFIXES$2 = ['bc1q', 'tb1q', 'bcrt1q'];
function check$2(address) {
    for (const prefix of VALID_PREFIXES$2) {
        if (address.startsWith(prefix)) {
            return true;
        }
    }
    return false;
}
function encode$4(input, network = 'main') {
    const prefix = BECH32_PREFIXES[network];
    const bytes = Buff$1.bytes(input);
    checkSize(bytes, 20);
    return bytes.toBech32(prefix, 0);
}
function decode$4(address) {
    if (!check$2(address)) {
        throw new TypeError('Invalid segwit address!');
    }
    return Buff$1.bech32(address);
}
function scriptPubKey$2(input) {
    const bytes = Buff$1.bytes(input);
    checkSize(bytes, 20);
    return ['OP_0', bytes.hex];
}
function fromPubKey$1(pubkey, network) {
    const bytes = Buff$1.bytes(pubkey);
    checkSize(bytes, 33);
    const hash = bytes.toHash('hash160');
    return encode$4(hash, network);
}
const P2WPKH = { check: check$2, encode: encode$4, decode: decode$4, scriptPubKey: scriptPubKey$2, fromPubKey: fromPubKey$1 };

const VALID_PREFIXES$1 = ['bc1q', 'tb1q', 'bcrt1q'];
function check$1(address) {
    for (const prefix of VALID_PREFIXES$1) {
        if (address.startsWith(prefix)) {
            return true;
        }
    }
    return false;
}
function encode$3(input, network = 'main') {
    const prefix = BECH32_PREFIXES[network];
    const bytes = Buff$1.bytes(input);
    checkSize(bytes, 32);
    return bytes.toBech32(prefix, 0);
}
function decode$3(address) {
    if (!check$1(address)) {
        throw new TypeError('Invalid segwit address!');
    }
    return Buff$1.bech32(address);
}
function scriptPubKey$1(input) {
    const bytes = Buff$1.bytes(input);
    checkSize(bytes, 32);
    return ['OP_0', bytes.hex];
}
function fromScript(script, network) {
    const bytes = Script.fmt.toBytes(script, false);
    const hash = bytes.toHash('sha256');
    return encode$3(hash, network);
}
const P2WSH = { check: check$1, encode: encode$3, decode: decode$3, scriptPubKey: scriptPubKey$1, fromScript };

function xOnlyPub(key) {
    const bytes = Buff$1.bytes(key);
    return (bytes.length > 32) ? bytes.slice(1, 33) : bytes;
}

const VALID_PREFIXES = ['bc1p', 'tb1p', 'bcrt1p'];
function check(address) {
    for (const prefix of VALID_PREFIXES) {
        if (address.startsWith(prefix)) {
            return true;
        }
    }
    return false;
}
function encode$2(input, network = 'main') {
    const prefix = BECH32_PREFIXES[network];
    const bytes = Buff$1.bytes(input);
    checkSize(bytes, 32);
    return bytes.toBech32(prefix, 1);
}
function decode$2(address) {
    if (!check(address)) {
        throw new TypeError('Invalid taproot address!');
    }
    return Buff$1.bech32(address);
}
function scriptPubKey(input) {
    const bytes = Buff$1.bytes(input);
    checkSize(bytes, 32);
    return ['OP_1', bytes.hex];
}
function fromPubKey(pubkey, network) {
    const bytes = xOnlyPub(pubkey);
    return encode$2(bytes, network);
}
const P2TR = { check, encode: encode$2, decode: decode$2, scriptPubKey, fromPubKey };

function createTx(template) {
    const { version = 2, vin = [], vout = [], locktime = 0 } = template;
    const inputs = vin.map(e => {
        if (e.witness === undefined)
            e.witness = [];
        return e;
    });
    return { version, vin: inputs, vout, locktime };
}

function encodeTx(txdata, omitWitness) {
    const { version, vin, vout, locktime } = createTx(txdata);
    const useWitness = (omitWitness !== true && checkForWitness(vin));
    const raw = [encodeVersion(version)];
    if (useWitness) {
        raw.push(Buff$1.hex('0001'));
    }
    raw.push(encodeInputs(vin));
    raw.push(encodeOutputs(vout));
    for (const txin of vin) {
        if (useWitness) {
            raw.push(encodeWitness(txin.witness));
        }
    }
    raw.push(encodeLocktime(locktime));
    return Buff$1.join(raw);
}
function checkForWitness(vin) {
    for (const txin of vin) {
        const { witness } = txin;
        if (typeof witness === 'string' ||
            witness instanceof Uint8Array ||
            (Array.isArray(witness) && witness.length > 0)) {
            return true;
        }
    }
    return false;
}
function encodeVersion(num) {
    return Buff$1.num(num, 4).reverse();
}
function encodeTxid(txid) {
    return Buff$1.hex(txid, 32).reverse();
}
function encodePrevOut(vout) {
    return Buff$1.num(vout, 4).reverse();
}
function encodeSequence(sequence = 0xFFFFFFFF) {
    return Buff$1.bytes(sequence).reverse();
}
function encodeInputs(arr) {
    const raw = [Buff$1.varInt(arr.length)];
    for (const vin of arr) {
        const { txid, vout, scriptSig, sequence } = vin;
        raw.push(encodeTxid(txid));
        raw.push(encodePrevOut(vout));
        raw.push(encodeScript(scriptSig, true));
        raw.push(encodeSequence(sequence));
    }
    return Buff$1.join(raw);
}
function encodeValue(value) {
    if (typeof value === 'number') {
        if (value % 1 !== 0) {
            throw new Error('Value must be an integer:' + String(value));
        }
        return Buff$1.num(value, 8).reverse();
    }
    return Buff$1.big(value, 8).reverse();
}
function encodeOutputs(arr) {
    const raw = [Buff$1.varInt(arr.length)];
    for (const vout of arr) {
        raw.push(encodeOutput(vout));
    }
    return Buff$1.join(raw);
}
function encodeOutput(vout) {
    const { value, scriptPubKey } = vout;
    const raw = [];
    raw.push(encodeValue(value));
    raw.push(encodeScript(scriptPubKey, true));
    return Buff$1.join(raw);
}
function encodeWitness(data = []) {
    const buffer = [];
    if (Array.isArray(data)) {
        const count = Buff$1.varInt(data.length);
        buffer.push(count);
        for (const entry of data) {
            buffer.push(encodeData(entry));
        }
        return Buff$1.join(buffer);
    }
    else {
        return Buff$1.normalize(data);
    }
}
function encodeData(data) {
    return (!isEmpty(data))
        ? encodeScript(data, true)
        : new Buff$1(0);
}
function isEmpty(data) {
    if (Array.isArray(data)) {
        return data.length === 0;
    }
    if (typeof data === 'string') {
        if (data === '')
            return true;
    }
    const bytes = Buff$1.bytes(data);
    return bytes.length === 1 && bytes[0] === 0;
}
function encodeLocktime(locktime) {
    if (typeof locktime === 'string') {
        return Buff$1.hex(locktime, 4);
    }
    if (typeof locktime === 'number') {
        return Buff$1.num(locktime, 4).reverse();
    }
    throw new Error('Unrecognized format: ' + String(locktime));
}

function decodeTx(bytes) {
    if (typeof bytes === 'string') {
        bytes = Buff$1.hex(bytes).raw;
    }
    const stream = new Stream$1(bytes);
    const version = readVersion(stream);
    const hasWitness = checkWitnessFlag(stream);
    const vin = readInputs(stream);
    const vout = readOutputs(stream);
    if (hasWitness) {
        for (const txin of vin) {
            txin.witness = readWitness$1(stream);
        }
    }
    const locktime = readLocktime(stream);
    return { version, vin, vout, locktime };
}
function readVersion(stream) {
    return stream.read(4).reverse().toNum();
}
function checkWitnessFlag(stream) {
    const [marker, flag] = [...stream.peek(2)];
    if (marker === 0) {
        stream.read(2);
        if (flag === 1) {
            return true;
        }
        else {
            throw new Error(`Invalid witness flag: ${flag}`);
        }
    }
    return false;
}
function readInputs(stream) {
    const inputs = [];
    const vinCount = stream.readSize();
    for (let i = 0; i < vinCount; i++) {
        inputs.push(readInput(stream));
    }
    return inputs;
}
function readInput(stream) {
    return {
        txid: stream.read(32).reverse().toHex(),
        vout: stream.read(4).reverse().toNum(),
        scriptSig: readScript(stream, true),
        sequence: stream.read(4).reverse().toHex(),
        witness: []
    };
}
function readOutputs(stream) {
    const outputs = [];
    const outcount = stream.readSize();
    for (let i = 0; i < outcount; i++) {
        outputs.push(readOutput(stream));
    }
    return outputs;
}
function readOutput(stream) {
    return {
        value: stream.read(8).reverse().big,
        scriptPubKey: readScript(stream, true)
    };
}
function readWitness$1(stream) {
    const stack = [];
    const count = stream.readSize();
    for (let i = 0; i < count; i++) {
        const word = readData(stream, true);
        stack.push(word ?? '');
    }
    return stack;
}
function readData(stream, varint) {
    const size = (varint === true)
        ? stream.readSize('be')
        : stream.size;
    return size > 0
        ? stream.read(size).hex
        : null;
}
function readScript(stream, varint) {
    const data = readData(stream, varint);
    return (data !== null) ? data : [];
}
function readLocktime(stream) {
    return stream.read(4).reverse().toNum();
}

function toJson(txdata) {
    if (isBytes(txdata)) {
        return decodeTx(txdata);
    }
    if (typeof txdata === 'object' &&
        !(txdata instanceof Uint8Array)) {
        encodeTx(txdata);
        return createTx(txdata);
    }
    throw new Error('Invalid format: ' + String(typeof txdata));
}
function toBytes(txdata) {
    if (isBytes(txdata)) {
        decodeTx(txdata);
        return Buff$1.bytes(txdata);
    }
    if (typeof txdata === 'object') {
        return encodeTx(txdata);
    }
    throw new Error('Invalid format: ' + String(typeof txdata));
}
const TxFmt = {
    toBytes,
    toJson
};

const OUTPUT_TYPES = [
    ['p2pkh', /^76a914(?<hash>\w{40})88ac$/],
    ['p2sh', /^a914(?<hash>\w{40})87$/],
    ['p2w-pkh', /^0014(?<hash>\w{40})$/],
    ['p2w-sh', /^0020(?<hash>\w{64})$/],
    ['p2tr', /^5120(?<hash>\w{64})$/]
];
const LEAF_VERSIONS = [
    0xc0, 0xc2, 0xc4, 0xc6, 0xc8, 0xca, 0xcc, 0xce,
    0xd0, 0xd2, 0xd4, 0xd6, 0xd8, 0xda, 0xdc, 0xde,
    0xe0, 0xe2, 0xe4, 0xe6, 0xe8, 0xea, 0xec, 0xee,
    0xf0, 0xf2, 0xf4, 0xf6, 0xf8, 0xfa, 0xfc, 0xfe,
    0x66, 0x7e, 0x80, 0x84, 0x96, 0x98, 0xba, 0xbc,
    0xbe
];
function parseAnnex(data) {
    let item = data.at(-1);
    if (isHex(item)) {
        item = Buff$1.hex(item);
    }
    if (data.length > 1 &&
        item instanceof Uint8Array &&
        item[0] === 0x50) {
        data.pop();
        return Buff$1.raw(item);
    }
    return null;
}
function parseBlock(data) {
    let item = data.at(-1);
    if (isHex(item)) {
        item = Buff$1.hex(item);
    }
    if (data.length > 1 &&
        item instanceof Uint8Array &&
        item.length > 32 &&
        LEAF_VERSIONS.includes(item[0] & 0xfe)) {
        data.pop();
        return Buff$1.raw(item);
    }
    return null;
}
function parseWitScript(data) {
    if (data.length > 1) {
        const item = data.at(-1);
        try {
            const script = Script.fmt.toBytes(item);
            data.pop();
            return script;
        }
        catch (err) {
            return null;
        }
    }
    return null;
}
function parseParams(data) {
    const params = [];
    for (const d of data) {
        if (isHex(d) ||
            d instanceof Uint8Array) {
            params.push(d);
        }
    }
    return params;
}
function readWitness(data = []) {
    const items = [...data];
    const annex = parseAnnex(items);
    const cblock = parseBlock(items);
    const script = parseWitScript(items);
    const params = parseParams(items);
    return { annex, cblock, script, params };
}
function readScriptPubKey(script) {
    const hex = Script.fmt.toBytes(script, false).hex;
    for (const [keytype, pattern] of OUTPUT_TYPES) {
        const type = keytype;
        const { groups } = pattern.exec(hex) ?? {};
        const { hash } = groups ?? {};
        if (isHex(hash)) {
            return { type, data: Buff$1.hex(hash) };
        }
    }
    return { type: 'raw', data: Buff$1.hex(hex) };
}
function getTxid(txdata) {
    const data = TxFmt.toBytes(txdata);
    return data.toHash('hash256').reverse().hex;
}

const Tx = {
    create: createTx,
    encode: encodeTx,
    decode: decodeTx,
    fmt: TxFmt,
    util: {
        getTxid,
        readScriptPubKey,
        readWitness
    }
};

const ADDRESS_TYPES = [
    ['1', 'p2pkh', 'main', 20, 'base58'],
    ['3', 'p2sh', 'main', 20, 'base58'],
    ['m', 'p2pkh', 'testnet', 20, 'base58'],
    ['n', 'p2pkh', 'testnet', 20, 'base58'],
    ['2', 'p2sh', 'testnet', 20, 'base58'],
    ['bc1q', 'p2w-pkh', 'main', 20, 'bech32'],
    ['tb1q', 'p2w-pkh', 'testnet', 20, 'bech32'],
    ['bcrt1q', 'p2w-pkh', 'regtest', 20, 'bech32'],
    ['bc1q', 'p2w-sh', 'main', 32, 'bech32'],
    ['tb1q', 'p2w-sh', 'testnet', 32, 'bech32'],
    ['bcrt1q', 'p2w-sh', 'regtest', 32, 'bech32'],
    ['bc1p', 'p2tr', 'main', 32, 'bech32m'],
    ['tb1p', 'p2tr', 'testnet', 32, 'bech32m'],
    ['bcrt1p', 'p2tr', 'regtest', 32, 'bech32m']
];
function decodeFormat(address, format) {
    switch (format) {
        case 'base58': return Buff$1.b58chk(address).slice(1);
        case 'bech32': return Buff$1.bech32(address);
        case 'bech32m': return Buff$1.bech32(address);
        default: throw new Error('Invalid address format: ' + format);
    }
}
function getData(address) {
    for (const row of ADDRESS_TYPES) {
        const [prefix, _type, _network, size, format] = row;
        if (address.startsWith(prefix)) {
            const bytes = decodeFormat(address, format);
            if (bytes.length === size)
                return row;
        }
    }
    throw new Error('Invalid address: ' + address);
}
function getTool(type) {
    switch (type) {
        case 'p2pkh': return P2PKH;
        case 'p2sh': return P2SH;
        case 'p2w-pkh': return P2WPKH;
        case 'p2w-sh': return P2WSH;
        case 'p2tr': return P2TR;
        default: throw new Error('Invalid address type: ' + type);
    }
}
function decodeAddress(address) {
    const [prefix, type, network] = getData(address);
    const tool = getTool(type);
    const data = tool.decode(address, network);
    const script = tool.scriptPubKey(data);
    return { prefix, type, network, data, script };
}
function fromScriptPubKey(script, network) {
    const { type, data } = Tx.util.readScriptPubKey(script);
    const tool = getTool(type);
    return tool.encode(data, network);
}
function toScriptPubKey(address) {
    const { script } = decodeAddress(address);
    return Script.fmt.toAsm(script, false);
}

const Address = {
    p2pkh: P2PKH,
    p2sh: P2SH,
    p2wpkh: P2WPKH,
    p2wsh: P2WSH,
    p2tr: P2TR,
    decode: decodeAddress,
    fromScriptPubKey,
    toScriptPubKey
};

const VALID_HASH_TYPES$1 = [0x01, 0x02, 0x03];
function hashTx$1(txdata, idx, config = {}) {
    const { sigflag = 0x01 } = config;
    const isAnypay = (sigflag & 0x80) === 0x80;
    const flag = sigflag % 0x80;
    if (!VALID_HASH_TYPES$1.includes(flag)) {
        throw new Error('Invalid hash type: ' + String(sigflag));
    }
    const tx = Tx.fmt.toJson(txdata);
    const { version, vin, vout, locktime } = tx;
    const { txid, vout: prevIdx, prevout, sequence } = vin[idx];
    const { value } = prevout ?? {};
    if (value === undefined) {
        throw new Error('Prevout value is empty!');
    }
    let script = config.script;
    if (script === undefined &&
        config.pubkey !== undefined) {
        const pkhash = Buff$1.bytes(config.pubkey).toHash('hash160');
        script = `76a914${pkhash.hex}88ac`;
    }
    if (script === undefined) {
        throw new Error('No pubkey / script has been set!');
    }
    if (Script.fmt.toAsm(script).includes('OP_CODESEPARATOR')) {
        throw new Error('This library does not currently support the use of OP_CODESEPARATOR in segwit scripts.');
    }
    const sighash = [
        encodeVersion(version),
        hashPrevouts(vin, isAnypay),
        hashSequence$1(vin, flag, isAnypay),
        encodeTxid(txid),
        encodePrevOut(prevIdx),
        Script.encode(script, true),
        encodeValue(value),
        encodeSequence(sequence),
        hashOutputs$1(vout, idx, flag),
        encodeLocktime(locktime),
        Buff$1.num(sigflag, 4).reverse()
    ];
    return Buff$1.join(sighash).toHash('hash256');
}
function hashPrevouts(vin, isAnypay) {
    if (isAnypay === true) {
        return Buff$1.num(0, 32);
    }
    const stack = [];
    for (const { txid, vout } of vin) {
        stack.push(encodeTxid(txid));
        stack.push(encodePrevOut(vout));
    }
    return Buff$1.join(stack).toHash('hash256');
}
function hashSequence$1(vin, sigflag, isAnyPay) {
    if (isAnyPay || sigflag !== 0x01) {
        return Buff$1.num(0, 32);
    }
    const stack = [];
    for (const { sequence } of vin) {
        stack.push(encodeSequence(sequence));
    }
    return Buff$1.join(stack).toHash('hash256');
}
function hashOutputs$1(vout, idx, sigflag) {
    const stack = [];
    if (sigflag === 0x01) {
        for (const { value, scriptPubKey } of vout) {
            stack.push(encodeValue(value));
            stack.push(Script.encode(scriptPubKey, true));
        }
        return Buff$1.join(stack).toHash('hash256');
    }
    if (sigflag === 0x03 && idx < vout.length) {
        const { value, scriptPubKey } = vout[idx];
        stack.push(encodeValue(value));
        stack.push(Script.encode(scriptPubKey, true));
        return Buff$1.join(stack).toHash('hash256');
    }
    return Buff$1.num(0, 32);
}

/*! noble-secp256k1 - MIT License (c) 2019 Paul Miller (paulmillr.com) */
const crypto$2 = getCryptoLib$1();
function getCryptoLib$1() {
    if (typeof globalThis !== 'undefined') {
        return globalThis.crypto;
    }
    if (typeof window !== 'undefined') {
        return window.crypto;
    }
    throw new Error('Must be using https in browser, or node/deno environment that supports globalthis.crypto object!');
}
const _0n = BigInt(0);
const _1n = BigInt(1);
const _2n = BigInt(2);
const _3n = BigInt(3);
const _8n = BigInt(8);
const CURVE = Object.freeze({
    a: _0n,
    b: BigInt(7),
    P: BigInt('0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f'),
    n: BigInt('0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141'),
    h: _1n,
    Gx: BigInt('55066263022277343669578718895168534326250603453777594175500187360389116729240'),
    Gy: BigInt('32670510020758816978083085130507043184471273380659243275938904335757337482424'),
    beta: BigInt('0x7ae96a2b657c07106e64479eac3434e99cf0497512f58995c1396c28719501ee'),
});
const divNearest = (a, b) => (a + b / _2n) / b;
const endo = {
    beta: BigInt('0x7ae96a2b657c07106e64479eac3434e99cf0497512f58995c1396c28719501ee'),
    splitScalar(k) {
        const { n } = CURVE;
        const a1 = BigInt('0x3086d221a7d46bcde86c90e49284eb15');
        const b1 = -_1n * BigInt('0xe4437ed6010e88286f547fa90abfe4c3');
        const a2 = BigInt('0x114ca50f7a8e2f3f657c1108d9d44cfd8');
        const b2 = a1;
        const POW_2_128 = BigInt('0x100000000000000000000000000000000');
        const c1 = divNearest(b2 * k, n);
        const c2 = divNearest(-b1 * k, n);
        let k1 = mod(k - c1 * a1 - c2 * a2, n);
        let k2 = mod(-c1 * b1 - c2 * b2, n);
        const k1neg = k1 > POW_2_128;
        const k2neg = k2 > POW_2_128;
        if (k1neg)
            k1 = n - k1;
        if (k2neg)
            k2 = n - k2;
        if (k1 > POW_2_128 || k2 > POW_2_128) {
            throw new Error('splitScalarEndo: Endomorphism failed, k=' + k);
        }
        return { k1neg, k1, k2neg, k2 };
    },
};
const fieldLen = 32;
const groupLen = 32;
const hashLen = 32;
const compressedLen = fieldLen + 1;
const uncompressedLen = 2 * fieldLen + 1;
function weierstrass(x) {
    const { a, b } = CURVE;
    const x2 = mod(x * x);
    const x3 = mod(x2 * x);
    return mod(x3 + a * x + b);
}
const USE_ENDOMORPHISM = CURVE.a === _0n;
class ShaError extends Error {
    constructor(message) {
        super(message);
    }
}
function assertJacPoint(other) {
    if (!(other instanceof JacobianPoint))
        throw new TypeError('JacobianPoint expected');
}
class JacobianPoint {
    constructor(x, y, z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }
    static get BASE() {
        return new JacobianPoint(CURVE.Gx, CURVE.Gy, _1n);
    }
    static get ZERO() {
        return new JacobianPoint(_0n, _1n, _0n);
    }
    static fromAffine(p) {
        if (!(p instanceof Point$1)) {
            throw new TypeError('JacobianPoint#fromAffine: expected Point');
        }
        if (p.equals(Point$1.ZERO))
            return JacobianPoint.ZERO;
        return new JacobianPoint(p.x, p.y, _1n);
    }
    static toAffineBatch(points) {
        const toInv = invertBatch(points.map((p) => p.z));
        return points.map((p, i) => p.toAffine(toInv[i]));
    }
    static normalizeZ(points) {
        return JacobianPoint.toAffineBatch(points).map(JacobianPoint.fromAffine);
    }
    equals(other) {
        assertJacPoint(other);
        const { x: X1, y: Y1, z: Z1 } = this;
        const { x: X2, y: Y2, z: Z2 } = other;
        const Z1Z1 = mod(Z1 * Z1);
        const Z2Z2 = mod(Z2 * Z2);
        const U1 = mod(X1 * Z2Z2);
        const U2 = mod(X2 * Z1Z1);
        const S1 = mod(mod(Y1 * Z2) * Z2Z2);
        const S2 = mod(mod(Y2 * Z1) * Z1Z1);
        return U1 === U2 && S1 === S2;
    }
    negate() {
        return new JacobianPoint(this.x, mod(-this.y), this.z);
    }
    double() {
        const { x: X1, y: Y1, z: Z1 } = this;
        const A = mod(X1 * X1);
        const B = mod(Y1 * Y1);
        const C = mod(B * B);
        const x1b = X1 + B;
        const D = mod(_2n * (mod(x1b * x1b) - A - C));
        const E = mod(_3n * A);
        const F = mod(E * E);
        const X3 = mod(F - _2n * D);
        const Y3 = mod(E * (D - X3) - _8n * C);
        const Z3 = mod(_2n * Y1 * Z1);
        return new JacobianPoint(X3, Y3, Z3);
    }
    add(other) {
        assertJacPoint(other);
        const { x: X1, y: Y1, z: Z1 } = this;
        const { x: X2, y: Y2, z: Z2 } = other;
        if (X2 === _0n || Y2 === _0n)
            return this;
        if (X1 === _0n || Y1 === _0n)
            return other;
        const Z1Z1 = mod(Z1 * Z1);
        const Z2Z2 = mod(Z2 * Z2);
        const U1 = mod(X1 * Z2Z2);
        const U2 = mod(X2 * Z1Z1);
        const S1 = mod(mod(Y1 * Z2) * Z2Z2);
        const S2 = mod(mod(Y2 * Z1) * Z1Z1);
        const H = mod(U2 - U1);
        const r = mod(S2 - S1);
        if (H === _0n) {
            if (r === _0n) {
                return this.double();
            }
            else {
                return JacobianPoint.ZERO;
            }
        }
        const HH = mod(H * H);
        const HHH = mod(H * HH);
        const V = mod(U1 * HH);
        const X3 = mod(r * r - HHH - _2n * V);
        const Y3 = mod(r * (V - X3) - S1 * HHH);
        const Z3 = mod(Z1 * Z2 * H);
        return new JacobianPoint(X3, Y3, Z3);
    }
    subtract(other) {
        return this.add(other.negate());
    }
    multiplyUnsafe(scalar) {
        const P0 = JacobianPoint.ZERO;
        if (typeof scalar === 'bigint' && scalar === _0n)
            return P0;
        let n = normalizeScalar(scalar);
        if (n === _1n)
            return this;
        if (!USE_ENDOMORPHISM) {
            let p = P0;
            let d = this;
            while (n > _0n) {
                if (n & _1n)
                    p = p.add(d);
                d = d.double();
                n >>= _1n;
            }
            return p;
        }
        let { k1neg, k1, k2neg, k2 } = endo.splitScalar(n);
        let k1p = P0;
        let k2p = P0;
        let d = this;
        while (k1 > _0n || k2 > _0n) {
            if (k1 & _1n)
                k1p = k1p.add(d);
            if (k2 & _1n)
                k2p = k2p.add(d);
            d = d.double();
            k1 >>= _1n;
            k2 >>= _1n;
        }
        if (k1neg)
            k1p = k1p.negate();
        if (k2neg)
            k2p = k2p.negate();
        k2p = new JacobianPoint(mod(k2p.x * endo.beta), k2p.y, k2p.z);
        return k1p.add(k2p);
    }
    precomputeWindow(W) {
        const windows = USE_ENDOMORPHISM ? 128 / W + 1 : 256 / W + 1;
        const points = [];
        let p = this;
        let base = p;
        for (let window = 0; window < windows; window++) {
            base = p;
            points.push(base);
            for (let i = 1; i < 2 ** (W - 1); i++) {
                base = base.add(p);
                points.push(base);
            }
            p = base.double();
        }
        return points;
    }
    wNAF(n, affinePoint) {
        if (!affinePoint && this.equals(JacobianPoint.BASE))
            affinePoint = Point$1.BASE;
        const W = (affinePoint && affinePoint._WINDOW_SIZE) || 1;
        if (256 % W) {
            throw new Error('Point#wNAF: Invalid precomputation window, must be power of 2');
        }
        let precomputes = affinePoint && pointPrecomputes.get(affinePoint);
        if (!precomputes) {
            precomputes = this.precomputeWindow(W);
            if (affinePoint && W !== 1) {
                precomputes = JacobianPoint.normalizeZ(precomputes);
                pointPrecomputes.set(affinePoint, precomputes);
            }
        }
        let p = JacobianPoint.ZERO;
        let f = JacobianPoint.BASE;
        const windows = 1 + (USE_ENDOMORPHISM ? 128 / W : 256 / W);
        const windowSize = 2 ** (W - 1);
        const mask = BigInt(2 ** W - 1);
        const maxNumber = 2 ** W;
        const shiftBy = BigInt(W);
        for (let window = 0; window < windows; window++) {
            const offset = window * windowSize;
            let wbits = Number(n & mask);
            n >>= shiftBy;
            if (wbits > windowSize) {
                wbits -= maxNumber;
                n += _1n;
            }
            const offset1 = offset;
            const offset2 = offset + Math.abs(wbits) - 1;
            const cond1 = window % 2 !== 0;
            const cond2 = wbits < 0;
            if (wbits === 0) {
                f = f.add(constTimeNegate(cond1, precomputes[offset1]));
            }
            else {
                p = p.add(constTimeNegate(cond2, precomputes[offset2]));
            }
        }
        return { p, f };
    }
    multiply(scalar, affinePoint) {
        let n = normalizeScalar(scalar);
        let point;
        let fake;
        if (USE_ENDOMORPHISM) {
            const { k1neg, k1, k2neg, k2 } = endo.splitScalar(n);
            let { p: k1p, f: f1p } = this.wNAF(k1, affinePoint);
            let { p: k2p, f: f2p } = this.wNAF(k2, affinePoint);
            k1p = constTimeNegate(k1neg, k1p);
            k2p = constTimeNegate(k2neg, k2p);
            k2p = new JacobianPoint(mod(k2p.x * endo.beta), k2p.y, k2p.z);
            point = k1p.add(k2p);
            fake = f1p.add(f2p);
        }
        else {
            const { p, f } = this.wNAF(n, affinePoint);
            point = p;
            fake = f;
        }
        return JacobianPoint.normalizeZ([point, fake])[0];
    }
    toAffine(invZ) {
        const { x, y, z } = this;
        const is0 = this.equals(JacobianPoint.ZERO);
        if (invZ == null)
            invZ = is0 ? _8n : invert(z);
        const iz1 = invZ;
        const iz2 = mod(iz1 * iz1);
        const iz3 = mod(iz2 * iz1);
        const ax = mod(x * iz2);
        const ay = mod(y * iz3);
        const zz = mod(z * iz1);
        if (is0)
            return Point$1.ZERO;
        if (zz !== _1n)
            throw new Error('invZ was invalid');
        return new Point$1(ax, ay);
    }
}
function constTimeNegate(condition, item) {
    const neg = item.negate();
    return condition ? neg : item;
}
const pointPrecomputes = new WeakMap();
let Point$1 = class Point {
    constructor(x, y) {
        this.x = x;
        this.y = y;
    }
    _setWindowSize(windowSize) {
        this._WINDOW_SIZE = windowSize;
        pointPrecomputes.delete(this);
    }
    static get BASE() {
        return new Point$1(CURVE.Gx, CURVE.Gy);
    }
    static get ZERO() {
        return new Point$1(_0n, _0n);
    }
    hasEvenY() {
        return this.y % _2n === _0n;
    }
    static fromCompressedHex(bytes) {
        const isShort = bytes.length === 32;
        const x = bytesToNumber(isShort ? bytes : bytes.subarray(1));
        if (!isValidFieldElement(x))
            throw new Error('Point is not on curve');
        const y2 = weierstrass(x);
        let y = sqrtMod(y2);
        const isYOdd = (y & _1n) === _1n;
        if (isShort) {
            if (isYOdd)
                y = mod(-y);
        }
        else {
            const isFirstByteOdd = (bytes[0] & 1) === 1;
            if (isFirstByteOdd !== isYOdd)
                y = mod(-y);
        }
        const point = new Point$1(x, y);
        point.assertValidity();
        return point;
    }
    static fromUncompressedHex(bytes) {
        const x = bytesToNumber(bytes.subarray(1, fieldLen + 1));
        const y = bytesToNumber(bytes.subarray(fieldLen + 1, fieldLen * 2 + 1));
        const point = new Point$1(x, y);
        point.assertValidity();
        return point;
    }
    static fromHex(hex) {
        const bytes = ensureBytes(hex);
        const len = bytes.length;
        const header = bytes[0];
        if (len === fieldLen)
            return this.fromCompressedHex(bytes);
        if (len === compressedLen && (header === 0x02 || header === 0x03)) {
            return this.fromCompressedHex(bytes);
        }
        if (len === uncompressedLen && header === 0x04)
            return this.fromUncompressedHex(bytes);
        throw new Error(`Point.fromHex: received invalid point. Expected 32-${compressedLen} compressed bytes or ${uncompressedLen} uncompressed bytes, not ${len}`);
    }
    static fromPrivateKey(privateKey) {
        return Point$1.BASE.multiply(normalizePrivateKey(privateKey));
    }
    static fromSignature(msgHash, signature, recovery) {
        const { r, s } = normalizeSignature(signature);
        if (![0, 1, 2, 3].includes(recovery))
            throw new Error('Cannot recover: invalid recovery bit');
        const h = truncateHash(ensureBytes(msgHash));
        const { n } = CURVE;
        const radj = recovery === 2 || recovery === 3 ? r + n : r;
        const rinv = invert(radj, n);
        const u1 = mod(-h * rinv, n);
        const u2 = mod(s * rinv, n);
        const prefix = recovery & 1 ? '03' : '02';
        const R = Point$1.fromHex(prefix + numTo32bStr(radj));
        const Q = Point$1.BASE.multiplyAndAddUnsafe(R, u1, u2);
        if (!Q)
            throw new Error('Cannot recover signature: point at infinify');
        Q.assertValidity();
        return Q;
    }
    toRawBytes(isCompressed = false) {
        return hexToBytes$1(this.toHex(isCompressed));
    }
    toHex(isCompressed = false) {
        const x = numTo32bStr(this.x);
        if (isCompressed) {
            const prefix = this.hasEvenY() ? '02' : '03';
            return `${prefix}${x}`;
        }
        else {
            return `04${x}${numTo32bStr(this.y)}`;
        }
    }
    toHexX() {
        return this.toHex(true).slice(2);
    }
    toRawX() {
        return this.toRawBytes(true).slice(1);
    }
    assertValidity() {
        const msg = 'Point is not on elliptic curve';
        const { x, y } = this;
        if (!isValidFieldElement(x) || !isValidFieldElement(y))
            throw new Error(msg);
        const left = mod(y * y);
        const right = weierstrass(x);
        if (mod(left - right) !== _0n)
            throw new Error(msg);
    }
    equals(other) {
        return this.x === other.x && this.y === other.y;
    }
    negate() {
        return new Point$1(this.x, mod(-this.y));
    }
    double() {
        return JacobianPoint.fromAffine(this).double().toAffine();
    }
    add(other) {
        return JacobianPoint.fromAffine(this).add(JacobianPoint.fromAffine(other)).toAffine();
    }
    subtract(other) {
        return this.add(other.negate());
    }
    multiply(scalar) {
        return JacobianPoint.fromAffine(this).multiply(scalar, this).toAffine();
    }
    multiplyAndAddUnsafe(Q, a, b) {
        const P = JacobianPoint.fromAffine(this);
        const aP = a === _0n || a === _1n || this !== Point$1.BASE ? P.multiplyUnsafe(a) : P.multiply(a);
        const bQ = JacobianPoint.fromAffine(Q).multiplyUnsafe(b);
        const sum = aP.add(bQ);
        return sum.equals(JacobianPoint.ZERO) ? undefined : sum.toAffine();
    }
};
function sliceDER(s) {
    return Number.parseInt(s[0], 16) >= 8 ? '00' + s : s;
}
function parseDERInt(data) {
    if (data.length < 2 || data[0] !== 0x02) {
        throw new Error(`Invalid signature integer tag: ${bytesToHex$1(data)}`);
    }
    const len = data[1];
    const res = data.subarray(2, len + 2);
    if (!len || res.length !== len) {
        throw new Error(`Invalid signature integer: wrong length`);
    }
    if (res[0] === 0x00 && res[1] <= 0x7f) {
        throw new Error('Invalid signature integer: trailing length');
    }
    return { data: bytesToNumber(res), left: data.subarray(len + 2) };
}
function parseDERSignature(data) {
    if (data.length < 2 || data[0] != 0x30) {
        throw new Error(`Invalid signature tag: ${bytesToHex$1(data)}`);
    }
    if (data[1] !== data.length - 2) {
        throw new Error('Invalid signature: incorrect length');
    }
    const { data: r, left: sBytes } = parseDERInt(data.subarray(2));
    const { data: s, left: rBytesLeft } = parseDERInt(sBytes);
    if (rBytesLeft.length) {
        throw new Error(`Invalid signature: left bytes after parsing: ${bytesToHex$1(rBytesLeft)}`);
    }
    return { r, s };
}
class Signature {
    constructor(r, s) {
        this.r = r;
        this.s = s;
        this.assertValidity();
    }
    static fromCompact(hex) {
        const arr = hex instanceof Uint8Array;
        const name = 'Signature.fromCompact';
        if (typeof hex !== 'string' && !arr)
            throw new TypeError(`${name}: Expected string or Uint8Array`);
        const str = arr ? bytesToHex$1(hex) : hex;
        if (str.length !== 128)
            throw new Error(`${name}: Expected 64-byte hex`);
        return new Signature(hexToNumber(str.slice(0, 64)), hexToNumber(str.slice(64, 128)));
    }
    static fromDER(hex) {
        const arr = hex instanceof Uint8Array;
        if (typeof hex !== 'string' && !arr)
            throw new TypeError(`Signature.fromDER: Expected string or Uint8Array`);
        const { r, s } = parseDERSignature(arr ? hex : hexToBytes$1(hex));
        return new Signature(r, s);
    }
    static fromHex(hex) {
        return this.fromDER(hex);
    }
    assertValidity() {
        const { r, s } = this;
        if (!isWithinCurveOrder(r))
            throw new Error('Invalid Signature: r must be 0 < r < n');
        if (!isWithinCurveOrder(s))
            throw new Error('Invalid Signature: s must be 0 < s < n');
    }
    hasHighS() {
        const HALF = CURVE.n >> _1n;
        return this.s > HALF;
    }
    normalizeS() {
        return this.hasHighS() ? new Signature(this.r, mod(-this.s, CURVE.n)) : this;
    }
    toDERRawBytes() {
        return hexToBytes$1(this.toDERHex());
    }
    toDERHex() {
        const sHex = sliceDER(numberToHexUnpadded(this.s));
        const rHex = sliceDER(numberToHexUnpadded(this.r));
        const sHexL = sHex.length / 2;
        const rHexL = rHex.length / 2;
        const sLen = numberToHexUnpadded(sHexL);
        const rLen = numberToHexUnpadded(rHexL);
        const length = numberToHexUnpadded(rHexL + sHexL + 4);
        return `30${length}02${rLen}${rHex}02${sLen}${sHex}`;
    }
    toRawBytes() {
        return this.toDERRawBytes();
    }
    toHex() {
        return this.toDERHex();
    }
    toCompactRawBytes() {
        return hexToBytes$1(this.toCompactHex());
    }
    toCompactHex() {
        return numTo32bStr(this.r) + numTo32bStr(this.s);
    }
}
function concatBytes(...arrays) {
    if (!arrays.every((b) => b instanceof Uint8Array))
        throw new Error('Uint8Array list expected');
    if (arrays.length === 1)
        return arrays[0];
    const length = arrays.reduce((a, arr) => a + arr.length, 0);
    const result = new Uint8Array(length);
    for (let i = 0, pad = 0; i < arrays.length; i++) {
        const arr = arrays[i];
        result.set(arr, pad);
        pad += arr.length;
    }
    return result;
}
const hexes = Array.from({ length: 256 }, (_v, i) => i.toString(16).padStart(2, '0'));
function bytesToHex$1(uint8a) {
    if (!(uint8a instanceof Uint8Array))
        throw new Error('Expected Uint8Array');
    let hex = '';
    for (let i = 0; i < uint8a.length; i++) {
        hex += hexes[uint8a[i]];
    }
    return hex;
}
const POW_2_256 = BigInt('0x10000000000000000000000000000000000000000000000000000000000000000');
function numTo32bStr(num) {
    if (typeof num !== 'bigint')
        throw new Error('Expected bigint');
    if (!(_0n <= num && num < POW_2_256))
        throw new Error('Expected number 0 <= n < 2^256');
    return num.toString(16).padStart(64, '0');
}
function numTo32b(num) {
    const b = hexToBytes$1(numTo32bStr(num));
    if (b.length !== 32)
        throw new Error('Error: expected 32 bytes');
    return b;
}
function numberToHexUnpadded(num) {
    const hex = num.toString(16);
    return hex.length & 1 ? `0${hex}` : hex;
}
function hexToNumber(hex) {
    if (typeof hex !== 'string') {
        throw new TypeError('hexToNumber: expected string, got ' + typeof hex);
    }
    return BigInt(`0x${hex}`);
}
function hexToBytes$1(hex) {
    if (typeof hex !== 'string') {
        throw new TypeError('hexToBytes: expected string, got ' + typeof hex);
    }
    if (hex.length % 2)
        throw new Error('hexToBytes: received invalid unpadded hex' + hex.length);
    const array = new Uint8Array(hex.length / 2);
    for (let i = 0; i < array.length; i++) {
        const j = i * 2;
        const hexByte = hex.slice(j, j + 2);
        const byte = Number.parseInt(hexByte, 16);
        if (Number.isNaN(byte) || byte < 0)
            throw new Error('Invalid byte sequence');
        array[i] = byte;
    }
    return array;
}
function bytesToNumber(bytes) {
    return hexToNumber(bytesToHex$1(bytes));
}
function ensureBytes(hex) {
    return hex instanceof Uint8Array ? Uint8Array.from(hex) : hexToBytes$1(hex);
}
function normalizeScalar(num) {
    if (typeof num === 'number' && Number.isSafeInteger(num) && num > 0)
        return BigInt(num);
    if (typeof num === 'bigint' && isWithinCurveOrder(num))
        return num;
    throw new TypeError('Expected valid private scalar: 0 < scalar < curve.n');
}
function mod(a, b = CURVE.P) {
    const result = a % b;
    return result >= _0n ? result : b + result;
}
function pow2(x, power) {
    const { P } = CURVE;
    let res = x;
    while (power-- > _0n) {
        res *= res;
        res %= P;
    }
    return res;
}
function sqrtMod(x) {
    const { P } = CURVE;
    const _6n = BigInt(6);
    const _11n = BigInt(11);
    const _22n = BigInt(22);
    const _23n = BigInt(23);
    const _44n = BigInt(44);
    const _88n = BigInt(88);
    const b2 = (x * x * x) % P;
    const b3 = (b2 * b2 * x) % P;
    const b6 = (pow2(b3, _3n) * b3) % P;
    const b9 = (pow2(b6, _3n) * b3) % P;
    const b11 = (pow2(b9, _2n) * b2) % P;
    const b22 = (pow2(b11, _11n) * b11) % P;
    const b44 = (pow2(b22, _22n) * b22) % P;
    const b88 = (pow2(b44, _44n) * b44) % P;
    const b176 = (pow2(b88, _88n) * b88) % P;
    const b220 = (pow2(b176, _44n) * b44) % P;
    const b223 = (pow2(b220, _3n) * b3) % P;
    const t1 = (pow2(b223, _23n) * b22) % P;
    const t2 = (pow2(t1, _6n) * b2) % P;
    const rt = pow2(t2, _2n);
    const xc = (rt * rt) % P;
    if (xc !== x)
        throw new Error('Cannot find square root');
    return rt;
}
function invert(number, modulo = CURVE.P) {
    if (number === _0n || modulo <= _0n) {
        throw new Error(`invert: expected positive integers, got n=${number} mod=${modulo}`);
    }
    let a = mod(number, modulo);
    let b = modulo;
    let x = _0n, u = _1n;
    while (a !== _0n) {
        const q = b / a;
        const r = b % a;
        const m = x - u * q;
        b = a, a = r, x = u, u = m;
    }
    const gcd = b;
    if (gcd !== _1n)
        throw new Error('invert: does not exist');
    return mod(x, modulo);
}
function invertBatch(nums, p = CURVE.P) {
    const scratch = new Array(nums.length);
    const lastMultiplied = nums.reduce((acc, num, i) => {
        if (num === _0n)
            return acc;
        scratch[i] = acc;
        return mod(acc * num, p);
    }, _1n);
    const inverted = invert(lastMultiplied, p);
    nums.reduceRight((acc, num, i) => {
        if (num === _0n)
            return acc;
        scratch[i] = mod(acc * scratch[i], p);
        return mod(acc * num, p);
    }, inverted);
    return scratch;
}
function bits2int_2(bytes) {
    const delta = bytes.length * 8 - groupLen * 8;
    const num = bytesToNumber(bytes);
    return delta > 0 ? num >> BigInt(delta) : num;
}
function truncateHash(hash, truncateOnly = false) {
    const h = bits2int_2(hash);
    if (truncateOnly)
        return h;
    const { n } = CURVE;
    return h >= n ? h - n : h;
}
let _sha256Sync;
let _hmacSha256Sync;
class HmacDrbg {
    constructor(hashLen, qByteLen) {
        this.hashLen = hashLen;
        this.qByteLen = qByteLen;
        if (typeof hashLen !== 'number' || hashLen < 2)
            throw new Error('hashLen must be a number');
        if (typeof qByteLen !== 'number' || qByteLen < 2)
            throw new Error('qByteLen must be a number');
        this.v = new Uint8Array(hashLen).fill(1);
        this.k = new Uint8Array(hashLen).fill(0);
        this.counter = 0;
    }
    hmac(...values) {
        return utils.hmacSha256(this.k, ...values);
    }
    hmacSync(...values) {
        return _hmacSha256Sync(this.k, ...values);
    }
    checkSync() {
        if (typeof _hmacSha256Sync !== 'function')
            throw new ShaError('hmacSha256Sync needs to be set');
    }
    incr() {
        if (this.counter >= 1000)
            throw new Error('Tried 1,000 k values for sign(), all were invalid');
        this.counter += 1;
    }
    async reseed(seed = new Uint8Array()) {
        this.k = await this.hmac(this.v, Uint8Array.from([0x00]), seed);
        this.v = await this.hmac(this.v);
        if (seed.length === 0)
            return;
        this.k = await this.hmac(this.v, Uint8Array.from([0x01]), seed);
        this.v = await this.hmac(this.v);
    }
    reseedSync(seed = new Uint8Array()) {
        this.checkSync();
        this.k = this.hmacSync(this.v, Uint8Array.from([0x00]), seed);
        this.v = this.hmacSync(this.v);
        if (seed.length === 0)
            return;
        this.k = this.hmacSync(this.v, Uint8Array.from([0x01]), seed);
        this.v = this.hmacSync(this.v);
    }
    async generate() {
        this.incr();
        let len = 0;
        const out = [];
        while (len < this.qByteLen) {
            this.v = await this.hmac(this.v);
            const sl = this.v.slice();
            out.push(sl);
            len += this.v.length;
        }
        return concatBytes(...out);
    }
    generateSync() {
        this.checkSync();
        this.incr();
        let len = 0;
        const out = [];
        while (len < this.qByteLen) {
            this.v = this.hmacSync(this.v);
            const sl = this.v.slice();
            out.push(sl);
            len += this.v.length;
        }
        return concatBytes(...out);
    }
}
function isWithinCurveOrder(num) {
    return _0n < num && num < CURVE.n;
}
function isValidFieldElement(num) {
    return _0n < num && num < CURVE.P;
}
function kmdToSig(kBytes, m, d, lowS = true) {
    const { n } = CURVE;
    const k = truncateHash(kBytes, true);
    if (!isWithinCurveOrder(k))
        return;
    const kinv = invert(k, n);
    const q = Point$1.BASE.multiply(k);
    const r = mod(q.x, n);
    if (r === _0n)
        return;
    const s = mod(kinv * mod(m + d * r, n), n);
    if (s === _0n)
        return;
    let sig = new Signature(r, s);
    let recovery = (q.x === sig.r ? 0 : 2) | Number(q.y & _1n);
    if (lowS && sig.hasHighS()) {
        sig = sig.normalizeS();
        recovery ^= 1;
    }
    return { sig, recovery };
}
function normalizePrivateKey(key) {
    let num;
    if (typeof key === 'bigint') {
        num = key;
    }
    else if (typeof key === 'number' && Number.isSafeInteger(key) && key > 0) {
        num = BigInt(key);
    }
    else if (typeof key === 'string') {
        if (key.length !== 2 * groupLen)
            throw new Error('Expected 32 bytes of private key');
        num = hexToNumber(key);
    }
    else if (key instanceof Uint8Array) {
        if (key.length !== groupLen)
            throw new Error('Expected 32 bytes of private key');
        num = bytesToNumber(key);
    }
    else {
        throw new TypeError('Expected valid private key');
    }
    if (!isWithinCurveOrder(num))
        throw new Error('Expected private key: 0 < key < n');
    return num;
}
function normalizePublicKey(publicKey) {
    if (publicKey instanceof Point$1) {
        publicKey.assertValidity();
        return publicKey;
    }
    else {
        return Point$1.fromHex(publicKey);
    }
}
function normalizeSignature(signature) {
    if (signature instanceof Signature) {
        signature.assertValidity();
        return signature;
    }
    try {
        return Signature.fromDER(signature);
    }
    catch (error) {
        return Signature.fromCompact(signature);
    }
}
function getPublicKey(privateKey, isCompressed = false) {
    return Point$1.fromPrivateKey(privateKey).toRawBytes(isCompressed);
}
function recoverPublicKey(msgHash, signature, recovery, isCompressed = false) {
    return Point$1.fromSignature(msgHash, signature, recovery).toRawBytes(isCompressed);
}
function isProbPub(item) {
    const arr = item instanceof Uint8Array;
    const str = typeof item === 'string';
    const len = (arr || str) && item.length;
    if (arr)
        return len === compressedLen || len === uncompressedLen;
    if (str)
        return len === compressedLen * 2 || len === uncompressedLen * 2;
    if (item instanceof Point$1)
        return true;
    return false;
}
function getSharedSecret(privateA, publicB, isCompressed = false) {
    if (isProbPub(privateA))
        throw new TypeError('getSharedSecret: first arg must be private key');
    if (!isProbPub(publicB))
        throw new TypeError('getSharedSecret: second arg must be public key');
    const b = normalizePublicKey(publicB);
    b.assertValidity();
    return b.multiply(normalizePrivateKey(privateA)).toRawBytes(isCompressed);
}
function bits2int(bytes) {
    const slice = bytes.length > fieldLen ? bytes.slice(0, fieldLen) : bytes;
    return bytesToNumber(slice);
}
function bits2octets(bytes) {
    const z1 = bits2int(bytes);
    const z2 = mod(z1, CURVE.n);
    return int2octets(z2 < _0n ? z1 : z2);
}
function int2octets(num) {
    return numTo32b(num);
}
function initSigArgs(msgHash, privateKey, extraEntropy) {
    if (msgHash == null)
        throw new Error(`sign: expected valid message hash, not "${msgHash}"`);
    const h1 = ensureBytes(msgHash);
    const d = normalizePrivateKey(privateKey);
    const seedArgs = [int2octets(d), bits2octets(h1)];
    if (extraEntropy != null) {
        if (extraEntropy === true)
            extraEntropy = utils.randomBytes(fieldLen);
        const e = ensureBytes(extraEntropy);
        if (e.length !== fieldLen)
            throw new Error(`sign: Expected ${fieldLen} bytes of extra data`);
        seedArgs.push(e);
    }
    const seed = concatBytes(...seedArgs);
    const m = bits2int(h1);
    return { seed, m, d };
}
function finalizeSig(recSig, opts) {
    const { sig, recovery } = recSig;
    const { der, recovered } = Object.assign({ canonical: true, der: true }, opts);
    const hashed = der ? sig.toDERRawBytes() : sig.toCompactRawBytes();
    return recovered ? [hashed, recovery] : hashed;
}
async function sign$1(msgHash, privKey, opts = {}) {
    const { seed, m, d } = initSigArgs(msgHash, privKey, opts.extraEntropy);
    const drbg = new HmacDrbg(hashLen, groupLen);
    await drbg.reseed(seed);
    let sig;
    while (!(sig = kmdToSig(await drbg.generate(), m, d, opts.canonical)))
        await drbg.reseed();
    return finalizeSig(sig, opts);
}
function signSync(msgHash, privKey, opts = {}) {
    const { seed, m, d } = initSigArgs(msgHash, privKey, opts.extraEntropy);
    const drbg = new HmacDrbg(hashLen, groupLen);
    drbg.reseedSync(seed);
    let sig;
    while (!(sig = kmdToSig(drbg.generateSync(), m, d, opts.canonical)))
        drbg.reseedSync();
    return finalizeSig(sig, opts);
}
const vopts = { strict: true };
function verify$1(signature, msgHash, publicKey, opts = vopts) {
    let sig;
    try {
        sig = normalizeSignature(signature);
        msgHash = ensureBytes(msgHash);
    }
    catch (error) {
        return false;
    }
    const { r, s } = sig;
    if (opts.strict && sig.hasHighS())
        return false;
    const h = truncateHash(msgHash);
    let P;
    try {
        P = normalizePublicKey(publicKey);
    }
    catch (error) {
        return false;
    }
    const { n } = CURVE;
    const sinv = invert(s, n);
    const u1 = mod(h * sinv, n);
    const u2 = mod(r * sinv, n);
    const R = Point$1.BASE.multiplyAndAddUnsafe(P, u1, u2);
    if (!R)
        return false;
    const v = mod(R.x, n);
    return v === r;
}
function schnorrChallengeFinalize(ch) {
    return mod(bytesToNumber(ch), CURVE.n);
}
class SchnorrSignature {
    constructor(r, s) {
        this.r = r;
        this.s = s;
        this.assertValidity();
    }
    static fromHex(hex) {
        const bytes = ensureBytes(hex);
        if (bytes.length !== 64)
            throw new TypeError(`SchnorrSignature.fromHex: expected 64 bytes, not ${bytes.length}`);
        const r = bytesToNumber(bytes.subarray(0, 32));
        const s = bytesToNumber(bytes.subarray(32, 64));
        return new SchnorrSignature(r, s);
    }
    assertValidity() {
        const { r, s } = this;
        if (!isValidFieldElement(r) || !isWithinCurveOrder(s))
            throw new Error('Invalid signature');
    }
    toHex() {
        return numTo32bStr(this.r) + numTo32bStr(this.s);
    }
    toRawBytes() {
        return hexToBytes$1(this.toHex());
    }
}
function schnorrGetPublicKey(privateKey) {
    return Point$1.fromPrivateKey(privateKey).toRawX();
}
class InternalSchnorrSignature {
    constructor(message, privateKey, auxRand = utils.randomBytes()) {
        if (message == null)
            throw new TypeError(`sign: Expected valid message, not "${message}"`);
        this.m = ensureBytes(message);
        const { x, scalar } = this.getScalar(normalizePrivateKey(privateKey));
        this.px = x;
        this.d = scalar;
        this.rand = ensureBytes(auxRand);
        if (this.rand.length !== 32)
            throw new TypeError('sign: Expected 32 bytes of aux randomness');
    }
    getScalar(priv) {
        const point = Point$1.fromPrivateKey(priv);
        const scalar = point.hasEvenY() ? priv : CURVE.n - priv;
        return { point, scalar, x: point.toRawX() };
    }
    initNonce(d, t0h) {
        return numTo32b(d ^ bytesToNumber(t0h));
    }
    finalizeNonce(k0h) {
        const k0 = mod(bytesToNumber(k0h), CURVE.n);
        if (k0 === _0n)
            throw new Error('sign: Creation of signature failed. k is zero');
        const { point: R, x: rx, scalar: k } = this.getScalar(k0);
        return { R, rx, k };
    }
    finalizeSig(R, k, e, d) {
        return new SchnorrSignature(R.x, mod(k + e * d, CURVE.n)).toRawBytes();
    }
    error() {
        throw new Error('sign: Invalid signature produced');
    }
    async calc() {
        const { m, d, px, rand } = this;
        const tag = utils.taggedHash;
        const t = this.initNonce(d, await tag(TAGS.aux, rand));
        const { R, rx, k } = this.finalizeNonce(await tag(TAGS.nonce, t, px, m));
        const e = schnorrChallengeFinalize(await tag(TAGS.challenge, rx, px, m));
        const sig = this.finalizeSig(R, k, e, d);
        if (!(await schnorrVerify(sig, m, px)))
            this.error();
        return sig;
    }
    calcSync() {
        const { m, d, px, rand } = this;
        const tag = utils.taggedHashSync;
        const t = this.initNonce(d, tag(TAGS.aux, rand));
        const { R, rx, k } = this.finalizeNonce(tag(TAGS.nonce, t, px, m));
        const e = schnorrChallengeFinalize(tag(TAGS.challenge, rx, px, m));
        const sig = this.finalizeSig(R, k, e, d);
        if (!schnorrVerifySync(sig, m, px))
            this.error();
        return sig;
    }
}
async function schnorrSign(msg, privKey, auxRand) {
    return new InternalSchnorrSignature(msg, privKey, auxRand).calc();
}
function schnorrSignSync(msg, privKey, auxRand) {
    return new InternalSchnorrSignature(msg, privKey, auxRand).calcSync();
}
function initSchnorrVerify(signature, message, publicKey) {
    const raw = signature instanceof SchnorrSignature;
    const sig = raw ? signature : SchnorrSignature.fromHex(signature);
    if (raw)
        sig.assertValidity();
    return {
        ...sig,
        m: ensureBytes(message),
        P: normalizePublicKey(publicKey),
    };
}
function finalizeSchnorrVerify(r, P, s, e) {
    const R = Point$1.BASE.multiplyAndAddUnsafe(P, normalizePrivateKey(s), mod(-e, CURVE.n));
    if (!R || !R.hasEvenY() || R.x !== r)
        return false;
    return true;
}
async function schnorrVerify(signature, message, publicKey) {
    try {
        const { r, s, m, P } = initSchnorrVerify(signature, message, publicKey);
        const e = schnorrChallengeFinalize(await utils.taggedHash(TAGS.challenge, numTo32b(r), P.toRawX(), m));
        return finalizeSchnorrVerify(r, P, s, e);
    }
    catch (error) {
        return false;
    }
}
function schnorrVerifySync(signature, message, publicKey) {
    try {
        const { r, s, m, P } = initSchnorrVerify(signature, message, publicKey);
        const e = schnorrChallengeFinalize(utils.taggedHashSync(TAGS.challenge, numTo32b(r), P.toRawX(), m));
        return finalizeSchnorrVerify(r, P, s, e);
    }
    catch (error) {
        if (error instanceof ShaError)
            throw error;
        return false;
    }
}
const schnorr = {
    Signature: SchnorrSignature,
    getPublicKey: schnorrGetPublicKey,
    sign: schnorrSign,
    verify: schnorrVerify,
    signSync: schnorrSignSync,
    verifySync: schnorrVerifySync,
};
Point$1.BASE._setWindowSize(8);
const TAGS = {
    challenge: 'BIP0340/challenge',
    aux: 'BIP0340/aux',
    nonce: 'BIP0340/nonce',
};
const TAGGED_HASH_PREFIXES = {};
const utils = {
    bytesToHex: bytesToHex$1,
    hexToBytes: hexToBytes$1,
    concatBytes,
    mod,
    invert,
    isValidPrivateKey(privateKey) {
        try {
            normalizePrivateKey(privateKey);
            return true;
        }
        catch (error) {
            return false;
        }
    },
    _bigintTo32Bytes: numTo32b,
    _normalizePrivateKey: normalizePrivateKey,
    hashToPrivateKey: (hash) => {
        hash = ensureBytes(hash);
        const minLen = groupLen + 8;
        if (hash.length < minLen || hash.length > 1024) {
            throw new Error(`Expected valid bytes of private key as per FIPS 186`);
        }
        const num = mod(bytesToNumber(hash), CURVE.n - _1n) + _1n;
        return numTo32b(num);
    },
    randomBytes: (bytesLength = 32) => {
        return crypto$2.getRandomValues(new Uint8Array(bytesLength));
    },
    randomPrivateKey: () => utils.hashToPrivateKey(utils.randomBytes(groupLen + 8)),
    precompute(windowSize = 8, point = Point$1.BASE) {
        const cached = point === Point$1.BASE ? point : new Point$1(point.x, point.y);
        cached._setWindowSize(windowSize);
        cached.multiply(_3n);
        return cached;
    },
    sha256: async (...messages) => {
        const buffer = await crypto$2.subtle.digest('SHA-256', concatBytes(...messages));
        return new Uint8Array(buffer);
    },
    hmacSha256: async (key, ...messages) => {
        const ckey = await crypto$2.subtle.importKey('raw', key, { name: 'HMAC', hash: { name: 'SHA-256' } }, false, ['sign']);
        const message = concatBytes(...messages);
        const buffer = await crypto$2.subtle.sign('HMAC', ckey, message);
        return new Uint8Array(buffer);
    },
    sha256Sync: undefined,
    hmacSha256Sync: undefined,
    taggedHash: async (tag, ...messages) => {
        let tagP = TAGGED_HASH_PREFIXES[tag];
        if (tagP === undefined) {
            const tagH = await utils.sha256(Uint8Array.from(tag, (c) => c.charCodeAt(0)));
            tagP = concatBytes(tagH, tagH);
            TAGGED_HASH_PREFIXES[tag] = tagP;
        }
        return utils.sha256(tagP, ...messages);
    },
    taggedHashSync: (tag, ...messages) => {
        if (typeof _sha256Sync !== 'function')
            throw new ShaError('sha256Sync is undefined, you need to set it');
        let tagP = TAGGED_HASH_PREFIXES[tag];
        if (tagP === undefined) {
            const tagH = _sha256Sync(Uint8Array.from(tag, (c) => c.charCodeAt(0)));
            tagP = concatBytes(tagH, tagH);
            TAGGED_HASH_PREFIXES[tag] = tagP;
        }
        return _sha256Sync(tagP, ...messages);
    },
    _JacobianPoint: JacobianPoint,
};
Object.defineProperties(utils, {
    sha256Sync: {
        configurable: false,
        get() {
            return _sha256Sync;
        },
        set(val) {
            if (!_sha256Sync)
                _sha256Sync = val;
        },
    },
    hmacSha256Sync: {
        configurable: false,
        get() {
            return _hmacSha256Sync;
        },
        set(val) {
            if (!_hmacSha256Sync)
                _hmacSha256Sync = val;
        },
    },
});

var module = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CURVE: CURVE,
    Point: Point$1,
    Signature: Signature,
    getPublicKey: getPublicKey,
    getSharedSecret: getSharedSecret,
    recoverPublicKey: recoverPublicKey,
    schnorr: schnorr,
    sign: sign$1,
    signSync: signSync,
    utils: utils,
    verify: verify$1
});

function joinArray(arr) {
    let i, idx = 0;
    const size = arr.reduce((prev, curr) => prev + curr.length, 0);
    const buff = new Uint8Array(size);
    for (const bytes of arr) {
        for (i = 0; i < bytes.length; idx++, i++) {
            buff[idx] = bytes[i];
        }
    }
    return buff;
}

const digestLength = 32;
const blockSize = 64;
const K = new Uint32Array([
    0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5, 0x3956c25b,
    0x59f111f1, 0x923f82a4, 0xab1c5ed5, 0xd807aa98, 0x12835b01,
    0x243185be, 0x550c7dc3, 0x72be5d74, 0x80deb1fe, 0x9bdc06a7,
    0xc19bf174, 0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc,
    0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da, 0x983e5152,
    0xa831c66d, 0xb00327c8, 0xbf597fc7, 0xc6e00bf3, 0xd5a79147,
    0x06ca6351, 0x14292967, 0x27b70a85, 0x2e1b2138, 0x4d2c6dfc,
    0x53380d13, 0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
    0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3, 0xd192e819,
    0xd6990624, 0xf40e3585, 0x106aa070, 0x19a4c116, 0x1e376c08,
    0x2748774c, 0x34b0bcb5, 0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f,
    0x682e6ff3, 0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208,
    0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2
]);
function hashBlocks(w, v, p, pos, len) {
    let a, b, c, d, e, f, g, h, u, i, j, t1, t2;
    while (len >= 64) {
        a = v[0];
        b = v[1];
        c = v[2];
        d = v[3];
        e = v[4];
        f = v[5];
        g = v[6];
        h = v[7];
        for (i = 0; i < 16; i++) {
            j = pos + i * 4;
            w[i] = (((p[j] & 0xff) << 24) | ((p[j + 1] & 0xff) << 16) |
                ((p[j + 2] & 0xff) << 8) | (p[j + 3] & 0xff));
        }
        for (i = 16; i < 64; i++) {
            u = w[i - 2];
            t1 = (u >>> 17 | u << (32 - 17)) ^ (u >>> 19 | u << (32 - 19)) ^ (u >>> 10);
            u = w[i - 15];
            t2 = (u >>> 7 | u << (32 - 7)) ^ (u >>> 18 | u << (32 - 18)) ^ (u >>> 3);
            w[i] = (t1 + w[i - 7] | 0) + (t2 + w[i - 16] | 0);
        }
        for (i = 0; i < 64; i++) {
            t1 = (((((e >>> 6 | e << (32 - 6)) ^ (e >>> 11 | e << (32 - 11)) ^
                (e >>> 25 | e << (32 - 25))) + ((e & f) ^ (~e & g))) | 0) +
                ((h + ((K[i] + w[i]) | 0)) | 0)) | 0;
            t2 = (((a >>> 2 | a << (32 - 2)) ^ (a >>> 13 | a << (32 - 13)) ^
                (a >>> 22 | a << (32 - 22))) + ((a & b) ^ (a & c) ^ (b & c))) | 0;
            h = g;
            g = f;
            f = e;
            e = (d + t1) | 0;
            d = c;
            c = b;
            b = a;
            a = (t1 + t2) | 0;
        }
        v[0] += a;
        v[1] += b;
        v[2] += c;
        v[3] += d;
        v[4] += e;
        v[5] += f;
        v[6] += g;
        v[7] += h;
        pos += 64;
        len -= 64;
    }
    return pos;
}
let Hash$1 = class Hash {
    constructor() {
        this.digestLength = digestLength;
        this.blockSize = blockSize;
        this.state = new Int32Array(8);
        this.temp = new Int32Array(64);
        this.buffer = new Uint8Array(128);
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        this.reset();
    }
    reset() {
        this.state[0] = 0x6a09e667;
        this.state[1] = 0xbb67ae85;
        this.state[2] = 0x3c6ef372;
        this.state[3] = 0xa54ff53a;
        this.state[4] = 0x510e527f;
        this.state[5] = 0x9b05688c;
        this.state[6] = 0x1f83d9ab;
        this.state[7] = 0x5be0cd19;
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        return this;
    }
    clean() {
        for (let i = 0; i < this.buffer.length; i++) {
            this.buffer[i] = 0;
        }
        for (let i = 0; i < this.temp.length; i++) {
            this.temp[i] = 0;
        }
        this.reset();
    }
    update(data, dataLength = data.length) {
        if (this.finished) {
            throw new Error("SHA256: can't update because hash was finished.");
        }
        let dataPos = 0;
        this.bytesHashed += dataLength;
        if (this.bufferLength > 0) {
            while (this.bufferLength < 64 && dataLength > 0) {
                this.buffer[this.bufferLength++] = data[dataPos++];
                dataLength--;
            }
            if (this.bufferLength === 64) {
                hashBlocks(this.temp, this.state, this.buffer, 0, 64);
                this.bufferLength = 0;
            }
        }
        if (dataLength >= 64) {
            dataPos = hashBlocks(this.temp, this.state, data, dataPos, dataLength);
            dataLength %= 64;
        }
        while (dataLength > 0) {
            this.buffer[this.bufferLength++] = data[dataPos++];
            dataLength--;
        }
        return this;
    }
    finish(out) {
        if (!this.finished) {
            const bytesHashed = this.bytesHashed;
            const left = this.bufferLength;
            const bitLenHi = (bytesHashed / 0x20000000) | 0;
            const bitLenLo = bytesHashed << 3;
            const padLength = (bytesHashed % 64 < 56) ? 64 : 128;
            this.buffer[left] = 0x80;
            for (let i = left + 1; i < padLength - 8; i++) {
                this.buffer[i] = 0;
            }
            this.buffer[padLength - 8] = (bitLenHi >>> 24) & 0xff;
            this.buffer[padLength - 7] = (bitLenHi >>> 16) & 0xff;
            this.buffer[padLength - 6] = (bitLenHi >>> 8) & 0xff;
            this.buffer[padLength - 5] = (bitLenHi >>> 0) & 0xff;
            this.buffer[padLength - 4] = (bitLenLo >>> 24) & 0xff;
            this.buffer[padLength - 3] = (bitLenLo >>> 16) & 0xff;
            this.buffer[padLength - 2] = (bitLenLo >>> 8) & 0xff;
            this.buffer[padLength - 1] = (bitLenLo >>> 0) & 0xff;
            hashBlocks(this.temp, this.state, this.buffer, 0, padLength);
            this.finished = true;
        }
        for (let i = 0; i < 8; i++) {
            out[i * 4 + 0] = (this.state[i] >>> 24) & 0xff;
            out[i * 4 + 1] = (this.state[i] >>> 16) & 0xff;
            out[i * 4 + 2] = (this.state[i] >>> 8) & 0xff;
            out[i * 4 + 3] = (this.state[i] >>> 0) & 0xff;
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
    _saveState(out) {
        for (let i = 0; i < this.state.length; i++) {
            out[i] = this.state[i];
        }
    }
    _restoreState(from, bytesHashed) {
        for (let i = 0; i < this.state.length; i++) {
            this.state[i] = from[i];
        }
        this.bytesHashed = bytesHashed;
        this.finished = false;
        this.bufferLength = 0;
    }
};
class HMAC {
    constructor(key) {
        this.inner = new Hash$1();
        this.outer = new Hash$1();
        this.blockSize = this.inner.blockSize;
        this.digestLength = this.inner.digestLength;
        const pad = new Uint8Array(this.blockSize);
        if (key.length > this.blockSize) {
            (new Hash$1()).update(key).finish(pad).clean();
        }
        else {
            for (let i = 0; i < key.length; i++) {
                pad[i] = key[i];
            }
        }
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36;
        }
        this.inner.update(pad);
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36 ^ 0x5c;
        }
        this.outer.update(pad);
        this.istate = new Uint32Array(8);
        this.ostate = new Uint32Array(8);
        this.inner._saveState(this.istate);
        this.outer._saveState(this.ostate);
        for (let i = 0; i < pad.length; i++) {
            pad[i] = 0;
        }
    }
    reset() {
        this.inner._restoreState(this.istate, this.inner.blockSize);
        this.outer._restoreState(this.ostate, this.outer.blockSize);
        return this;
    }
    clean() {
        for (let i = 0; i < this.istate.length; i++) {
            this.ostate[i] = this.istate[i] = 0;
        }
        this.inner.clean();
        this.outer.clean();
    }
    update(data) {
        this.inner.update(data);
        return this;
    }
    finish(out) {
        if (this.outer.finished) {
            this.outer.finish(out);
        }
        else {
            this.inner.finish(out);
            this.outer.update(out, this.digestLength).finish(out);
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
}
function sha256$1(data) {
    const h = (new Hash$1()).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}
function hash256$1(data) {
    return sha256$1(sha256$1(data));
}
function hmac256$1(key, data) {
    const h = (new HMAC(key)).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}

const ec$2 = new TextEncoder();
const ALPHABETS = [
    {
        name: 'base58',
        charset: '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
    }
];
function getAlphabet(name) {
    for (const alpha of ALPHABETS) {
        if (alpha.name === name) {
            return alpha.charset;
        }
    }
    throw TypeError('Charset does not exist: ' + name);
}
function encode$1(data, charset, padding = false) {
    if (typeof data === 'string')
        data = ec$2.encode(data);
    const alphabet = getAlphabet(charset);
    const len = alphabet.length;
    const d = [];
    let s = '', i, j = 0, c, n;
    for (i = 0; i < data.length; i++) {
        j = 0;
        c = data[i];
        s += (c > 0 || (s.length ^ i) > 0) ? '' : '1';
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * 256 + c : c;
            c = n / len | 0;
            d[j] = n % len;
            j++;
        }
    }
    while (j-- > 0) {
        s += alphabet[d[j]];
    }
    return (padding && s.length % 4 > 0)
        ? s + '='.repeat(4 - s.length % 4)
        : s;
}
function decode$1(encoded, charset) {
    const alphabet = getAlphabet(charset);
    const len = alphabet.length, d = [], b = [];
    encoded = encoded.replace('=', '');
    let i, j = 0, c, n;
    for (i = 0; i < encoded.length; i++) {
        j = 0;
        c = alphabet.indexOf(encoded[i]);
        if (c < 0) {
            throw new Error(`Character range out of bounds: ${c}`);
        }
        if (!(c > 0 || (b.length ^ i) > 0))
            b.push(0);
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * len + c : c;
            c = n >> 8;
            d[j] = n % 256;
            j++;
        }
    }
    while (j-- > 0) {
        b.push(d[j]);
    }
    return new Uint8Array(b);
}
function addChecksum(data) {
    const sum = hash256$1(data);
    return joinArray([data, sum.slice(0, 4)]);
}
function checkTheSum(data) {
    const ret = data.slice(0, -4);
    const chk = data.slice(-4);
    const sum = hash256$1(ret).slice(0, 4);
    if (sum.toString() !== chk.toString()) {
        throw new Error('Invalid checksum!');
    }
    return ret;
}
const BaseX = {
    encode: encode$1,
    decode: decode$1
};
const Base58C = {
    encode: (data) => {
        const withSum = addChecksum(data);
        return BaseX.encode(withSum, 'base58');
    },
    decode: (data) => {
        const decoded = BaseX.decode(data, 'base58');
        return checkTheSum(decoded);
    }
};

const CHARSET = 'qpzry9x8gf2tvdw0s3jn54khce6mua7l';
const GENERATOR = [0x3b6a57b2, 0x26508e6d, 0x1ea119fa, 0x3d4233dd, 0x2a1462b3];
const ENCODINGS = [
    { version: 0, name: 'bech32', const: 1 },
    { version: 1, name: 'bech32m', const: 0x2bc830a3 }
];
function polymod(values) {
    let chk = 1;
    for (let p = 0; p < values.length; ++p) {
        const top = chk >> 25;
        chk = (chk & 0x1ffffff) << 5 ^ values[p];
        for (let i = 0; i < 5; ++i) {
            if (((top >> i) & 1) !== 0) {
                chk ^= GENERATOR[i];
            }
        }
    }
    return chk;
}
function hrpExpand(hrp) {
    const ret = [];
    let p;
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) >> 5);
    }
    ret.push(0);
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) & 31);
    }
    return ret;
}
function verifyChecksum(hrp, data, enc) {
    const combined = hrpExpand(hrp).concat(data);
    return polymod(combined) === enc.const;
}
function createChecksum(hrp, data, enc) {
    const values = hrpExpand(hrp).concat(data).concat([0, 0, 0, 0, 0, 0]);
    const mod = polymod(values) ^ enc.const;
    const ret = [];
    for (let p = 0; p < 6; ++p) {
        ret.push((mod >> 5 * (5 - p)) & 31);
    }
    return ret;
}
function convertBits(data, fromBits, toBits, pad = true) {
    const ret = [];
    let acc = 0;
    let bits = 0;
    const maxVal = (1 << toBits) - 1;
    const maxAcc = (1 << (fromBits + toBits - 1)) - 1;
    for (const val of data) {
        if (val < 0 || (val >> fromBits) > 0) {
            throw new Error('Failed to perform base conversion. Invalid value: ' + String(val));
        }
        acc = ((acc << fromBits) | val) & maxAcc;
        bits += fromBits;
        while (bits >= toBits) {
            bits -= toBits;
            ret.push((acc >> bits) & maxVal);
        }
    }
    if (pad) {
        if (bits > 0) {
            ret.push((acc << (toBits - bits)) & maxVal);
        }
    }
    else if (bits >= fromBits || ((acc << (toBits - bits)) & maxVal) > 0) {
        throw new Error('Failed to perform base conversion. Invalid Size!');
    }
    return ret;
}
function encode(hrp, data, enc) {
    const combined = data.concat(createChecksum(hrp, data, enc));
    let ret = hrp + '1';
    for (let p = 0; p < combined.length; ++p) {
        ret += CHARSET.charAt(combined[p]);
    }
    return ret;
}
function decode(bechstr) {
    if (!checkBounds(bechstr)) {
        throw new Error('Encoded string goes out of bounds!');
    }
    bechstr = bechstr.toLowerCase();
    if (!checkSeparatorPos(bechstr)) {
        throw new Error('Encoded string has invalid separator!');
    }
    const data = [];
    const pos = bechstr.lastIndexOf('1');
    const hrp = bechstr.substring(0, pos);
    for (let p = pos + 1; p < bechstr.length; ++p) {
        const d = CHARSET.indexOf(bechstr.charAt(p));
        if (d === -1) {
            throw new Error('Character idx out of bounds: ' + String(p));
        }
        data.push(d);
    }
    const enc = ENCODINGS.find(e => e.version === data[0]) ?? ENCODINGS[0];
    if (!verifyChecksum(hrp, data, enc)) {
        throw new Error('Checksum verification failed!');
    }
    return [hrp, data.slice(0, data.length - 6)];
}
function checkBounds(bechstr) {
    let p;
    let char;
    let hasLower = false;
    let hasUpper = false;
    for (p = 0; p < bechstr.length; ++p) {
        char = bechstr.charCodeAt(p);
        if (char < 33 || char > 126) {
            return false;
        }
        if (char >= 97 && char <= 122) {
            hasLower = true;
        }
        if (char >= 65 && char <= 90) {
            hasUpper = true;
        }
    }
    if (hasLower && hasUpper)
        return false;
    return true;
}
function checkSeparatorPos(bechstr) {
    const pos = bechstr.lastIndexOf('1');
    return !(pos < 1 ||
        pos + 7 > bechstr.length ||
        bechstr.length > 90);
}
function b32encode(data, hrp = 'bc', version = 0) {
    const dat = [version, ...convertBits([...data], 8, 5)];
    const enc = ENCODINGS.find(e => e.version === version) ?? ENCODINGS[0];
    const str = encode(hrp, dat, enc);
    b32decode(str);
    return str;
}
function b32decode(str) {
    str = str.toLowerCase();
    const hrp = str.split('1', 1)[0];
    const [hrpgot, data] = decode(str);
    const decoded = convertBits(data.slice(1), 5, 8, false);
    const length = decoded.length;
    switch (true) {
        case (hrp !== hrpgot):
            throw new Error('Returned hrp string is invalid.');
        case (decoded === null || length < 2 || length > 40):
            throw new Error('Decoded string is invalid or out of spec.');
        case (data[0] > 16):
            throw new Error('Returned version bit is out of range.');
        default:
            return Uint8Array.from(decoded);
    }
}
function getVersion$1(str) {
    str = str.toLowerCase();
    const [_, data] = decode(str);
    return data[0];
}
const Bech32 = {
    encode: b32encode,
    decode: b32decode,
    version: getVersion$1
};

const BASE64_MAP = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
const B64URL_MAP = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_';
const ec$1 = new TextEncoder();
function b64encode(input, urlSafe = false) {
    if (typeof input === 'string')
        input = ec$1.encode(input);
    const map = urlSafe ? B64URL_MAP : BASE64_MAP;
    let output = '';
    let bits = 0;
    let buffer = 0;
    for (let i = 0; i < input.length; i++) {
        buffer = (buffer << 8) | input[i];
        bits += 8;
        while (bits >= 6) {
            bits -= 6;
            output += map[(buffer >> bits) & 0x3f];
        }
    }
    if (bits > 0) {
        buffer <<= 6 - bits;
        output += map[buffer & 0x3f];
        while (bits < 6) {
            output += urlSafe ? '' : '=';
            bits += 2;
        }
    }
    return output;
}
function b64decode(input, urlSafe = false) {
    const map = (urlSafe || input.includes('-') || input.includes('_'))
        ? B64URL_MAP.split('')
        : BASE64_MAP.split('');
    input = input.replace(/=+$/, '');
    const chars = input.split('');
    let bits = 0;
    let value = 0;
    const bytes = [];
    for (let i = 0; i < chars.length; i++) {
        const c = chars[i];
        const index = map.indexOf(c);
        if (index === -1) {
            throw new Error('Invalid character: ' + c);
        }
        bits += 6;
        value <<= 6;
        value |= index;
        if (bits >= 8) {
            bits -= 8;
            bytes.push((value >>> bits) & 0xff);
        }
    }
    return new Uint8Array(bytes);
}
const Base64 = {
    encode: b64encode,
    decode: b64decode
};
const B64URL = {
    encode: (data) => b64encode(data, true),
    decode: (data) => b64decode(data, true)
};

const ec = new TextEncoder();
const dc = new TextDecoder();
function strToBytes(str) {
    return ec.encode(str);
}
function hexToBytes(str) {
    const bytes = new Uint8Array(str.length / 2);
    let i, idx = 0;
    if (str.match(/[^a-fA-f0-9]/) !== null) {
        throw new TypeError('Invalid hex string: ' + str);
    }
    if (str.length % 2 > 0) {
        throw new Error(`Hex string length is uneven: ${str.length}`);
    }
    for (i = 0; i < str.length; i += 2) {
        bytes[idx] = parseInt(str.slice(i, i + 2), 16);
        idx += 1;
    }
    return bytes;
}
function numToBytes(num) {
    if (num === 0)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xff;
        bytes.push(byte);
        num = (num - byte) / 256;
    }
    return new Uint8Array(bytes);
}
function binaryToBytes(binary) {
    if (typeof binary === 'string') {
        binary = binary.split('').map(Number);
    }
    else if (!Array.isArray(binary)) {
        throw new Error('Invalid input type: expected a string or an array of numbers.');
    }
    if (binary.length % 8 !== 0) {
        throw new Error(`Binary array is invalid length: ${binary.length}`);
    }
    const bytes = new Uint8Array(binary.length / 8);
    for (let i = 0, ct = 0; i < binary.length; i += 8, ct++) {
        let byte = 0;
        for (let j = 0; j < 8; j++) {
            byte |= (binary[i + j] << (7 - j));
        }
        bytes[ct] = byte;
    }
    return bytes;
}
function bytesToBinary(bytes) {
    const bin = new Array(bytes.length * 8);
    let count = 0;
    for (const num of bytes) {
        if (num > 255) {
            throw new Error(`Invalid byte value: ${num}. Byte values must be between 0 and 255.`);
        }
        for (let i = 7; i >= 0; i--, count++) {
            bin[count] = (num >> i) & 1;
        }
    }
    return bin;
}
function bigToBytes(big) {
    if (big === 0n)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (big > 0n) {
        const byte = big & 0xffn;
        bytes.push(Number(byte));
        big = (big - byte) / 256n;
    }
    return new Uint8Array(bytes);
}
function bytesToStr(bytes) {
    return dc.decode(bytes);
}
function bytesToHex(bytes) {
    const chars = new Array(bytes.length);
    for (let i = 0; i < bytes.length; i++) {
        chars.push(bytes[i].toString(16).padStart(2, '0'));
    }
    return chars.join('');
}
function bytesToNum(bytes) {
    let num = 0, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256) + bytes[i];
    }
    return Number(num);
}
function bytesToBig(bytes) {
    let num = 0n, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256n) + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function buffer(value, bytes = true) {
    if (value instanceof ArrayBuffer) {
        return new Uint8Array(value);
    }
    if (value instanceof Uint8Array) {
        return value;
    }
    switch (typeof value) {
        case 'bigint':
            return bigToBytes(value);
        case 'boolean':
            return Uint8Array.of(value ? 1 : 0);
        case 'number':
            return numToBytes(value);
        case 'string':
            return (bytes)
                ? hexToBytes(value)
                : ec.encode(value);
        default:
            throw TypeError('Unsupported format:' + String(typeof value));
    }
}
function hexify(bytes) {
    bytes = buffer(bytes, true);
    return bytesToHex(bytes);
}
function serialize(data) {
    if (typeof data === 'object') {
        if (data instanceof Uint8Array) {
            return data;
        }
        try {
            return strToBytes(JSON.stringify(data));
        }
        catch {
            throw TypeError('Object is not serializable.');
        }
    }
    return buffer(data, false);
}
function revive(data) {
    if (data instanceof Uint8Array) {
        data = bytesToStr(data);
    }
    if (typeof data === 'string') {
        try {
            return JSON.parse(data);
        }
        catch {
            return data;
        }
    }
    return data;
}

function bytesToBigInt$1(bytes) {
    let num = 0n;
    for (let i = bytes.length - 1; i >= 0; i--) {
        num = num * 256n + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function bigIntToBytes$1(num, size = 0) {
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xffn;
        bytes.push(byte);
        num = (num - byte) / 256n;
    }
    let uint8 = Uint8Array.from(bytes.map((n) => Number(n)));
    if (size !== 0) {
        const resized = new Uint8Array(size);
        resized.set(uint8);
        uint8 = resized;
    }
    return uint8;
}
const ML$1 = [
    0n,
    1n,
    2n,
    3n,
    4n,
    5n,
    6n,
    7n,
    8n,
    9n,
    10n,
    11n,
    12n,
    13n,
    14n,
    15n,
    7n,
    4n,
    13n,
    1n,
    10n,
    6n,
    15n,
    3n,
    12n,
    0n,
    9n,
    5n,
    2n,
    14n,
    11n,
    8n,
    3n,
    10n,
    14n,
    4n,
    9n,
    15n,
    8n,
    1n,
    2n,
    7n,
    0n,
    6n,
    13n,
    11n,
    5n,
    12n,
    1n,
    9n,
    11n,
    10n,
    0n,
    8n,
    12n,
    4n,
    13n,
    3n,
    7n,
    15n,
    14n,
    5n,
    6n,
    2n,
    4n,
    0n,
    5n,
    9n,
    7n,
    12n,
    2n,
    10n,
    14n,
    1n,
    3n,
    8n,
    11n,
    6n,
    15n,
    13n
];
const MR$1 = [
    5n,
    14n,
    7n,
    0n,
    9n,
    2n,
    11n,
    4n,
    13n,
    6n,
    15n,
    8n,
    1n,
    10n,
    3n,
    12n,
    6n,
    11n,
    3n,
    7n,
    0n,
    13n,
    5n,
    10n,
    14n,
    15n,
    8n,
    12n,
    4n,
    9n,
    1n,
    2n,
    15n,
    5n,
    1n,
    3n,
    7n,
    14n,
    6n,
    9n,
    11n,
    8n,
    12n,
    2n,
    10n,
    0n,
    4n,
    13n,
    8n,
    6n,
    4n,
    1n,
    3n,
    11n,
    15n,
    0n,
    5n,
    12n,
    2n,
    13n,
    9n,
    7n,
    10n,
    14n,
    12n,
    15n,
    10n,
    4n,
    1n,
    5n,
    8n,
    7n,
    6n,
    2n,
    13n,
    14n,
    0n,
    3n,
    9n,
    11n
];
const RL$1 = [
    11n,
    14n,
    15n,
    12n,
    5n,
    8n,
    7n,
    9n,
    11n,
    13n,
    14n,
    15n,
    6n,
    7n,
    9n,
    8n,
    7n,
    6n,
    8n,
    13n,
    11n,
    9n,
    7n,
    15n,
    7n,
    12n,
    15n,
    9n,
    11n,
    7n,
    13n,
    12n,
    11n,
    13n,
    6n,
    7n,
    14n,
    9n,
    13n,
    15n,
    14n,
    8n,
    13n,
    6n,
    5n,
    12n,
    7n,
    5n,
    11n,
    12n,
    14n,
    15n,
    14n,
    15n,
    9n,
    8n,
    9n,
    14n,
    5n,
    6n,
    8n,
    6n,
    5n,
    12n,
    9n,
    15n,
    5n,
    11n,
    6n,
    8n,
    13n,
    12n,
    5n,
    12n,
    13n,
    14n,
    11n,
    8n,
    5n,
    6n
];
const RR$1 = [
    8n,
    9n,
    9n,
    11n,
    13n,
    15n,
    15n,
    5n,
    7n,
    7n,
    8n,
    11n,
    14n,
    14n,
    12n,
    6n,
    9n,
    13n,
    15n,
    7n,
    12n,
    8n,
    9n,
    11n,
    7n,
    7n,
    12n,
    7n,
    6n,
    15n,
    13n,
    11n,
    9n,
    7n,
    15n,
    11n,
    8n,
    6n,
    6n,
    14n,
    12n,
    13n,
    5n,
    14n,
    13n,
    13n,
    7n,
    5n,
    15n,
    5n,
    8n,
    11n,
    14n,
    14n,
    6n,
    14n,
    6n,
    9n,
    12n,
    9n,
    12n,
    5n,
    15n,
    8n,
    8n,
    5n,
    12n,
    9n,
    12n,
    5n,
    14n,
    6n,
    8n,
    13n,
    6n,
    5n,
    15n,
    13n,
    11n,
    11n
];
const KL$1 = [0n, 0x5a827999n, 0x6ed9eba1n, 0x8f1bbcdcn, 0xa953fd4en];
const KR$1 = [0x50a28be6n, 0x5c4dd124n, 0x6d703ef3n, 0x7a6d76e9n, 0n];
function fi$1(x, y, z, i) {
    switch (true) {
        case i === 0n:
            return x ^ y ^ z;
        case i === 1n:
            return (x & y) | (~x & z);
        case i === 2n:
            return (x | ~y) ^ z;
        case i === 3n:
            return (x & z) | (y & ~z);
        case i === 4n:
            return x ^ (y | ~z);
        default:
            throw new TypeError('Unknown I value: ' + String(i));
    }
}
function rol$1(x, i) {
    return ((x << i) | ((x & 0xffffffffn) >> (32n - i))) & 0xffffffffn;
}
function compress$1(h0, h1, h2, h3, h4, block) {
    const x = [];
    let rnd, elt, ert;
    let ml, kl, mr, kr;
    let al = h0, bl = h1, cl = h2, dl = h3, el = h4;
    let ar = h0, br = h1, cr = h2, dr = h3, er = h4;
    for (let i = 0; i < 16; i++) {
        const num = bytesToBigInt$1(block.slice(4 * i, 4 * (i + 1)));
        x.push(num);
    }
    for (let i = 0; i < 80; i++) {
        rnd = BigInt(i) >> 4n;
        ml = x[Number(ML$1[i])];
        kl = KL$1[Number(rnd)];
        mr = x[Number(MR$1[i])];
        kr = KR$1[Number(rnd)];
        al = rol$1(al + fi$1(bl, cl, dl, rnd) + ml + kl, RL$1[i]) + el;
        elt = el;
        el = dl;
        dl = rol$1(cl, 10n);
        cl = bl;
        bl = al;
        al = elt;
        ar = rol$1(ar + fi$1(br, cr, dr, 4n - rnd) + mr + kr, RR$1[i]) + er;
        ert = er;
        er = dr;
        dr = rol$1(cr, 10n);
        cr = br;
        br = ar;
        ar = ert;
    }
    return [h1 + cl + dr, h2 + dl + er, h3 + el + ar, h4 + al + br, h0 + bl + cr];
}
function ripemd160$1(bytes) {
    let state = [
        0x67452301n,
        0xefcdab89n,
        0x98badcfen,
        0x10325476n,
        0xc3d2e1f0n
    ];
    for (let b = 0; b < bytes.length >> 6; b++) {
        state = compress$1(...state, bytes.slice(64 * b, 64 * (b + 1)));
    }
    const zfill = new Array((119 - bytes.length) & 63).fill(0);
    const pad = [0x80, ...zfill];
    const fin = Uint8Array.from([
        ...bytes.slice(bytes.length & ~63),
        ...pad,
        ...bigIntToBytes$1(BigInt(8 * bytes.length), 8)
    ]);
    for (let i = 0; i < fin.length >> 6; i++) {
        state = compress$1(...state, fin.slice(64 * i, 64 * (i + 1)));
    }
    const ret = [];
    for (let i = 0; i < state.length; i++) {
        const num = state[i] & 0xffffffffn;
        ret.push(...bigIntToBytes$1(num, 4));
    }
    return Uint8Array.from(ret);
}

class PRNG {
    static seed(size = 32) {
        const date = numToBytes(Date.now());
        const seed = new Uint8Array(size * 4).fill(0);
        seed.map(() => Math.floor(Math.random() * 0xFFFF % 0xFF));
        return sha256$1(joinArray([seed, date]));
    }
    static getRandomValues(array) {
        if (array !== null) {
            const view = new Uint8Array(array.buffer);
            const size = view.length;
            const rand = PRNG.generate(size);
            for (let i = 0; i < size; i++) {
                view[i] = rand[i];
            }
            return array;
        }
        throw new Error('ArrayBufferView is null!');
    }
    static generate(size = 32) {
        let seed = PRNG.seed();
        const rnds = Math.ceil((size + 32) / 32);
        const pool = [];
        for (let i = 0; i < rnds; i++) {
            const prng = new PRNG(seed);
            const hash = prng.generate();
            seed = hash;
            pool.push(hash);
        }
        return joinArray(pool.reverse()).slice(0, size);
    }
    constructor(seed = PRNG.seed()) {
        this.x = bytesToBig(seed);
        this.y = BigInt(362436069);
        this.z = BigInt(521288629);
        this.w = BigInt(88675123);
        this.c = BigInt(0);
    }
    generate() {
        const next = this.next();
        if (next < (2n ** 512n)) {
            return this.generate();
        }
        return sha256$1(bigToBytes(next));
    }
    next() {
        this.x = BigInt(6906969069) * this.x + this.c;
        this.y ^= this.y << BigInt(13);
        this.y ^= this.y >> BigInt(17);
        this.y ^= this.y << BigInt(43);
        const t = this.z + this.w + this.c;
        this.z = this.w;
        this.c = t >> BigInt(63);
        this.w = t & BigInt('0x7fffffffffffffff');
        return this.x + this.y + this.w;
    }
}

const webcrypto = (typeof crypto !== 'undefined')
    ? crypto
    : (typeof globalThis?.crypto !== 'undefined')
        ? globalThis.crypto
        : (typeof window !== 'undefined')
            ? window.crypto
            : {
                getRandomValues: PRNG.getRandomValues,
                subtle: undefined
            };

class Buff extends Uint8Array {
    static { this.num = (number, size, endian = 'le') => {
        const b = new Buff(numToBytes(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static { this.big = (number, size, endian = 'le') => {
        const b = new Buff(bigToBytes(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static { this.bin = (data, size) => new Buff(binaryToBytes(data), size); }
    static { this.any = (data, size) => new Buff(buffer(data, false), size); }
    static { this.raw = (data, size) => new Buff(data, size); }
    static { this.str = (data, size) => new Buff(strToBytes(data), size); }
    static { this.hex = (data, size) => new Buff(hexToBytes(data), size); }
    static { this.json = (data) => new Buff(strToBytes(JSON.stringify(data))); }
    static { this.bytes = (data, size) => new Buff(buffer(data, true), size); }
    static { this.base64 = (data) => new Buff(Base64.decode(data)); }
    static { this.b64url = (data) => new Buff(B64URL.decode(data)); }
    static { this.bech32 = (data) => new Buff(Bech32.decode(data)); }
    static { this.b58chk = (data) => new Buff(Base58C.decode(data)); }
    constructor(data, size) {
        data = buffer(data, true);
        if (typeof size === 'number') {
            const tmp = new Uint8Array(size).fill(0);
            tmp.set(new Uint8Array(data));
            data = tmp.buffer;
        }
        super(data);
        return this;
    }
    get arr() {
        return [...this];
    }
    get num() {
        return this.toNum();
    }
    get big() {
        return this.toBig();
    }
    get str() {
        return this.toStr();
    }
    get hex() {
        return this.toHex();
    }
    get raw() {
        return new Uint8Array(this);
    }
    get bits() {
        return this.toBits();
    }
    get bin() {
        return this.toBin();
    }
    get b58chk() {
        return this.tob58chk();
    }
    get base64() {
        return this.toBase64();
    }
    get b64url() {
        return this.toB64url();
    }
    get digest() {
        return this.toHash();
    }
    get id() {
        return this.toHash().hex;
    }
    get stream() {
        return new Stream(this);
    }
    toNum(endian = 'le') {
        return (endian === 'le')
            ? bytesToNum(this.reverse())
            : bytesToNum(this);
    }
    toBig(endian = 'le') {
        return (endian === 'le')
            ? bytesToBig(this.reverse())
            : bytesToBig(this);
    }
    toHash(type = 'sha256') {
        switch (type) {
            case 'sha256':
                return new Buff(sha256$1(this));
            case 'hash256':
                return new Buff(sha256$1(sha256$1(this)));
            case 'ripe160':
                return new Buff(ripemd160$1(this));
            case 'hash160':
                return new Buff(ripemd160$1(sha256$1(this)));
            default:
                throw new Error('Unrecognized format:' + String(type));
        }
    }
    toHmac(key) {
        if (typeof key === 'string') {
            key = strToBytes(key);
        }
        return new Buff(hmac256$1(key, this));
    }
    toStr() { return bytesToStr(this); }
    toHex() { return bytesToHex(this); }
    toJson() { return JSON.parse(bytesToStr(this)); }
    toBytes() { return new Uint8Array(this); }
    toBits() { return bytesToBinary(this); }
    toBin() { return bytesToBinary(this).join(''); }
    tob58chk() { return Base58C.encode(this); }
    toB64url() { return B64URL.encode(this); }
    toBase64() { return Base64.encode(this); }
    toBech32(hrp, version = 0) { return Bech32.encode(this, hrp, version); }
    prepend(data) {
        return Buff.join([Buff.bytes(data), this]);
    }
    append(data) {
        return Buff.join([this, Buff.bytes(data)]);
    }
    slice(start, end) {
        return new Buff(new Uint8Array(this).slice(start, end));
    }
    reverse() {
        return new Buff(new Uint8Array(this).reverse());
    }
    write(bytes, offset) {
        this.set(bytes, offset);
    }
    prefixSize(endian) {
        const size = Buff.varInt(this.length, endian);
        return Buff.join([size, this]);
    }
    static from(data) {
        return new Buff(Uint8Array.from(data));
    }
    static of(...args) {
        return new Buff(Uint8Array.of(...args));
    }
    static join(arr) {
        const data = arr.map(e => Buff.bytes(e));
        return new Buff(joinArray(data));
    }
    static varInt(num, endian) {
        if (num < 0xFD) {
            return Buff.num(num, 1);
        }
        else if (num < 0x10000) {
            return Buff.of(0xFD, ...Buff.num(num, 2, endian));
        }
        else if (num < 0x100000000) {
            return Buff.of(0xFE, ...Buff.num(num, 4, endian));
        }
        else if (BigInt(num) < 0x10000000000000000n) {
            return Buff.of(0xFF, ...Buff.num(num, 8, endian));
        }
        else {
            throw new Error(`Value is too large: ${num}`);
        }
    }
    static { this.encode = strToBytes; }
    static { this.decode = bytesToStr; }
    static random(size = 32) {
        return new Buff(webcrypto.getRandomValues(new Uint8Array(size)));
    }
    static normalize(bytes, size) {
        return new Buff(buffer(bytes, true), size);
    }
    static hexify(bytes) {
        return hexify(bytes);
    }
    static serialize(data, size) {
        return new Buff(serialize(data), size);
    }
    static revive(data) {
        return revive(data);
    }
}
class Stream {
    constructor(data) {
        this.data = new Uint8Array(data);
        this.size = this.data.length;
    }
    peek(size) {
        if (size > this.size) {
            throw new Error(`Size greater than stream: ${size} > ${this.size}`);
        }
        return new Buff(this.data.slice(0, size).buffer);
    }
    read(size) {
        size = size ?? this.readSize();
        const chunk = this.peek(size);
        this.data = this.data.slice(size);
        this.size = this.data.length;
        return chunk;
    }
    readSize(endian) {
        const num = this.read(1).num;
        switch (true) {
            case (num >= 0 && num < 0xFD):
                return num;
            case (num === 0xFD):
                return this.read(2).toNum(endian);
            case (num === 0xFE):
                return this.read(4).toNum(endian);
            case (num === 0xFF):
                return this.read(8).toNum(endian);
            default:
                throw new Error(`Varint is out of range: ${num}`);
        }
    }
}

getCryptoLib();
function getCryptoLib() {
    if (typeof globalThis !== 'undefined' &&
        typeof globalThis.crypto !== 'undefined') {
        return globalThis.crypto;
    }
    if (typeof window !== 'undefined') {
        return window.crypto;
    }
    throw new Error('Must be using https in browser, or node/deno environment that supports globalthis.crypto object!');
}
function getXOnlyPub(bytes) {
    const b = Buff.bytes(bytes);
    if (b.length === 33) {
        return b.slice(1, 33);
    }
    if (b.length === 32) {
        return b;
    }
    throw new Error('Invalid key length: ' + String(b.length));
}

class Field extends Uint8Array {
    static { this.N = CURVE.n; }
    static mod(x, n = Field.N) {
        return utils.mod(x, n);
    }
    static pow(x, e, n = Field.N) {
        x = Field.mod(x, n);
        e = Field.mod(e, n);
        if (x === 0n)
            return 0n;
        let res = 1n;
        while (e > 0n) {
            if ((e & 1n) === 1n) {
                res = Field.mod(res * x, n);
            }
            e = e >> 1n;
            x = Field.mod(x * x, n);
        }
        return res;
    }
    static normalize(num) {
        num = normalizeField(num);
        num = Field.mod(num);
        Field.validate(num);
        return Buff.big(num, 32);
    }
    static validate(num) {
        return utils.isValidPrivateKey(num);
    }
    constructor(x) {
        super(Field.normalize(x), 32);
    }
    get buff() {
        return new Buff(this);
    }
    get raw() {
        return this.buff.raw;
    }
    get big() {
        return this.buff.big;
    }
    get hex() {
        return this.buff.hex;
    }
    get point() {
        return this.generate();
    }
    get xpoint() {
        return new Point(this.point.x);
    }
    get hasOddY() {
        return this.point.hasOddY;
    }
    get negated() {
        return (this.hasOddY)
            ? this.negate()
            : this;
    }
    gt(big) {
        const x = new Field(big);
        return x.big > this.big;
    }
    lt(big) {
        const x = new Field(big);
        return x.big < this.big;
    }
    eq(big) {
        const x = new Field(big);
        return x.big === this.big;
    }
    ne(big) {
        const x = new Field(big);
        return x.big !== this.big;
    }
    add(big) {
        const x = new Field(big);
        return new Field(this.big + x.big);
    }
    sub(big) {
        const x = new Field(big);
        return new Field(this.big - x.big);
    }
    mul(big) {
        const x = new Field(big);
        return new Field(this.big * x.big);
    }
    pow(big, n = Field.N - 1n) {
        const x = new Field(big);
        const e = Field.mod(x.big, n);
        return new Field(this.big ** e);
    }
    div(big) {
        const x = new Field(big);
        const d = this.pow(x.big, Field.N - 2n);
        return new Field(this.big * d.big);
    }
    negate() {
        return new Field(Field.N - this.big);
    }
    generate() {
        return Point.import(Point$1.BASE.multiply(this.big));
    }
}
class Point {
    static { this.N = CURVE.n; }
    static validate(x) {
        try {
            x = new Point(x);
            return true;
        }
        catch {
            return false;
        }
    }
    static normalize(x) {
        x = normalizePoint(x);
        return Point$1.fromHex(x);
    }
    static generate(big) {
        return new Field(big).generate();
    }
    static import(point) {
        const p = (point instanceof Point)
            ? { x: point.x.big, y: point.y.big }
            : point;
        return new Point(p.x, p.y);
    }
    constructor(x, y) {
        this.__p = (typeof x === 'bigint' &&
            typeof y === 'bigint')
            ? new Point$1(x, y)
            : Point.normalize(x);
        this.p.assertValidity();
    }
    get p() {
        return this.__p;
    }
    get x() {
        return Buff.big(this.p.x, 32);
    }
    get y() {
        return Buff.big(this.p.y, 32);
    }
    get buff() {
        const p = this.p.toRawBytes(true);
        if (p.length < 33) {
            const prefix = this.p.hasEvenY() ? 0x02 : 0x03;
            return Buff.join([prefix, p]);
        }
        else {
            return Buff.raw(p);
        }
    }
    get raw() {
        return this.buff.raw;
    }
    get hex() {
        return this.buff.hex;
    }
    get hasEvenY() {
        return this.p.hasEvenY();
    }
    get hasOddY() {
        return !this.p.hasEvenY();
    }
    eq(value) {
        return (value instanceof Point)
            ? this.p.equals(new Point$1(value.x.big, value.y.big))
            : (value instanceof Uint8Array)
                ? this.x.big === Buff.raw(value).big
                : (typeof value === 'number')
                    ? BigInt(value) === this.x.big
                    : value === this.x.big;
    }
    add(x) {
        return (x instanceof Point)
            ? Point.import(this.p.add(x.p))
            : Point.import(this.p.add(Point.generate(x).p));
    }
    sub(x) {
        return (x instanceof Point)
            ? Point.import(this.p.subtract(x.p))
            : Point.import(this.p.subtract(Point.generate(x).p));
    }
    mul(value) {
        return (value instanceof Point)
            ? Point.import(this.p.multiply(value.x.big))
            : Point.import(this.p.multiply(normalizeField(value)));
    }
    negate() {
        return Point.import(this.p.negate());
    }
}
function normalizeField(value) {
    if (value instanceof Field) {
        return value.big;
    }
    if (value instanceof Point) {
        return value.x.big;
    }
    if (value instanceof Uint8Array) {
        return Buff.raw(value).big;
    }
    if (typeof value === 'string') {
        return Buff.hex(value).big;
    }
    if (typeof value === 'number') {
        return Buff.num(value).big;
    }
    if (typeof value === 'bigint') {
        return BigInt(value);
    }
    throw TypeError('Invalid input type:' + typeof value);
}
function normalizePoint(value) {
    if (value instanceof Field) {
        return value.hex;
    }
    if (value instanceof Point) {
        return value.hex;
    }
    if (value instanceof Uint8Array) {
        return Buff.raw(value).hex;
    }
    if (typeof value === 'string') {
        return value;
    }
    if (typeof value === 'number') {
        return Buff.num(value).hex;
    }
    if (typeof value === 'bigint') {
        return Buff.big(value).hex;
    }
    throw TypeError('Invalid input type:' + typeof value);
}

async function sign$2(secret, message, type = 'taproot') {
    const msg = Buff.bytes(message).raw;
    const key = Buff.bytes(secret).raw;
    switch (type) {
        case 'ecdsa':
            return sign$1(msg, key);
        case 'taproot':
            return schnorr.sign(msg, key);
        default:
            throw new Error('Unknown signature type:' + String(type));
    }
}
async function verify$2(signature, message, pubkey, type = 'taproot') {
    const sig = Buff.bytes(signature).raw;
    const msg = Buff.bytes(message).raw;
    const pub = Buff.bytes(pubkey).raw;
    switch (type) {
        case 'ecdsa':
            return verify$1(sig, msg, pub);
        case 'taproot':
            return schnorr.verify(sig, msg, getXOnlyPub(pub));
        default:
            throw new Error('Unknown signature type:' + String(type));
    }
}

const DEFAULT_CONFIG = {
    type: 'ecdsa'
};
class SecretKey extends Uint8Array {
    static random(opt) {
        return new SecretKey(Buff.random(32), opt);
    }
    constructor(secret, options = {}) {
        super(new Field(secret));
        this.config = { ...DEFAULT_CONFIG, ...options };
        this.xonly = this.config.type === 'taproot';
    }
    get buff() {
        return new Buff(this);
    }
    get raw() {
        return this.buff.raw;
    }
    get hex() {
        return this.buff.hex;
    }
    get field() {
        return new Field(this);
    }
    get point() {
        return this.field.point;
    }
    get pub() {
        return new PublicKey(this.point.raw, this.config);
    }
    get hasEvenY() {
        return this.point.hasEvenY;
    }
    get hasOddY() {
        return this.point.hasOddY;
    }
    get xfilter() {
        return (this.xonly && this.hasOddY)
            ? this.negate()
            : this;
    }
    add(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.add(bytes), this.config);
    }
    sub(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.sub(bytes), this.config);
    }
    mul(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.mul(bytes), this.config);
    }
    div(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.div(bytes), this.config);
    }
    pow(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.pow(bytes), this.config);
    }
    negate() {
        return new SecretKey(this.field.negate(), this.config);
    }
    async sign(message, type = this.config.type) {
        return sign$2(this.raw, message, type);
    }
    async verify(signature, message, type = this.config.type) {
        return verify$2(signature, message, this.pub.raw, type);
    }
    toWIF(prefix = 0x80) {
        return Buff.join([prefix, this, 0x01]).b58chk;
    }
}
class PublicKey extends Uint8Array {
    static random(opt) {
        return SecretKey.random(opt).pub;
    }
    static fromSecret(bytes, opt) {
        return new SecretKey(bytes, opt).pub;
    }
    static { this.xfilter = getXOnlyPub; }
    constructor(pubkey, options = {}) {
        const config = { ...DEFAULT_CONFIG, ...options };
        if (config.type === 'taproot') {
            super(PublicKey.xfilter(pubkey), 32);
        }
        else {
            super(Buff.bytes(pubkey), 33);
        }
        this.config = { ...DEFAULT_CONFIG, ...options };
        this.xonly = config.type === 'taproot';
    }
    get buff() {
        return (this.xonly) ? this.x : new Buff(this);
    }
    get raw() {
        return this.buff.raw;
    }
    get hex() {
        return this.buff.hex;
    }
    get point() {
        return new Point(this);
    }
    get x() {
        return this.point.x;
    }
    get y() {
        return this.point.y;
    }
    get hasEvenY() {
        return this.point.hasEvenY;
    }
    get hasOddY() {
        return this.point.hasOddY;
    }
    add(bytes) {
        return new PublicKey(this.point.add(bytes).raw, this.config);
    }
    sub(bytes) {
        return new PublicKey(this.point.sub(bytes).raw, this.config);
    }
    mul(bytes) {
        return new PublicKey(this.point.mul(bytes).raw, this.config);
    }
    negate() {
        return new PublicKey(this.point.negate().raw, this.config);
    }
    async verify(signature, message, type = this.config.type) {
        return verify$2(signature, message, this.raw, type);
    }
}

async function signTx$1(seckey, txdata, index, config = {}) {
    const { sigflag = 0x01 } = config;
    const hash = hashTx$1(txdata, index, config);
    const sig = await module.sign(hash, seckey);
    return Buff$1.join([sig, sigflag]);
}

async function verifyTx$1(txdata, index, config = {}) {
    const tx = Tx.fmt.toJson(txdata);
    const { throws = false } = config;
    const { witness = [] } = tx.vin[index];
    const witnessData = Tx.util.readWitness(witness);
    const { script, params } = witnessData;
    let pub = null;
    if (params.length < 1) {
        return safeThrow('Invalid witness data: ' + String(witness), throws);
    }
    if (config.script === undefined &&
        script !== null) {
        config.script = script;
    }
    if (config.pubkey !== undefined) {
        pub = Buff$1.bytes(config.pubkey);
    }
    else if (params.length > 1 &&
        params[1].length === 33) {
        pub = Buff$1.bytes(params[1]);
    }
    else {
        return safeThrow('No pubkey provided!', throws);
    }
    const rawsig = Script.fmt.toParam(params[0]);
    const signature = rawsig.slice(0, -1);
    const sigflag = rawsig.slice(-1)[0];
    const hash = hashTx$1(tx, index, { ...config, sigflag });
    if (!module.verify(signature.hex, hash.hex, pub.hex)) {
        return safeThrow('Invalid signature!', throws);
    }
    return true;
}

const SWSigner = {
    hash: hashTx$1,
    sign: signTx$1,
    verify: verifyTx$1
};

const VALID_HASH_TYPES = [0x00, 0x01, 0x02, 0x03, 0x81, 0x82, 0x83];
function hashTx(template, index, config = {}) {
    const { extension, sigflag = 0x00, extflag = 0x00, key_version = 0x00, separator_pos = 0xFFFFFFFF } = config;
    const txdata = Tx.fmt.toJson(template);
    const { version, vin: input, vout: output, locktime } = txdata;
    if (index >= input.length) {
        throw new Error('Index out of bounds: ' + String(index));
    }
    if (!VALID_HASH_TYPES.includes(sigflag)) {
        throw new Error('Invalid hash type: ' + String(sigflag));
    }
    if (extflag < 0 || extflag > 127) {
        throw new Error('Extention flag out of range: ' + String(extflag));
    }
    const { txid, vout, sequence, witness = [] } = input[index];
    const isAnyPay = (sigflag & 0x80) === 0x80;
    const annex = getAnnexData(witness);
    const annexBit = (annex !== undefined) ? 1 : 0;
    const extendBit = (extension !== undefined) ? 1 : 0;
    const spendType = ((extflag + extendBit) * 2) + annexBit;
    const hashtag = Buff$1.str('TapSighash').digest;
    const preimage = [
        hashtag,
        hashtag,
        Buff$1.num(0x00, 1),
        Buff$1.num(sigflag, 1),
        encodeVersion(version),
        encodeLocktime(locktime)
    ];
    if (!isAnyPay) {
        const prevouts = input.map(e => getPrevout(e));
        preimage.push(hashOutpoints(input), hashAmounts(prevouts), hashScripts(prevouts), hashSequence(input));
    }
    if ((sigflag & 0x03) < 2 || (sigflag & 0x03) > 3) {
        preimage.push(hashOutputs(output));
    }
    preimage.push(Buff$1.num(spendType, 1));
    if (isAnyPay) {
        const { value, scriptPubKey } = getPrevout(input[index]);
        preimage.push(encodeTxid(txid), encodePrevOut(vout), encodeValue(value), Script.encode(scriptPubKey, true), encodeSequence(sequence));
    }
    else {
        preimage.push(Buff$1.num(index, 4).reverse());
    }
    if (annex !== undefined) {
        preimage.push(annex);
    }
    if ((sigflag & 0x03) === 0x03) {
        preimage.push(hashOutput(output[index]));
    }
    if (extension !== undefined) {
        preimage.push(Buff$1.bytes(extension), Buff$1.num(key_version), Buff$1.num(separator_pos, 4));
    }
    return Buff$1.join(preimage).digest;
}
function hashOutpoints(vin) {
    const stack = [];
    for (const { txid, vout } of vin) {
        stack.push(encodeTxid(txid));
        stack.push(encodePrevOut(vout));
    }
    return Buff$1.join(stack).digest;
}
function hashSequence(vin) {
    const stack = [];
    for (const { sequence } of vin) {
        stack.push(encodeSequence(sequence));
    }
    return Buff$1.join(stack).digest;
}
function hashAmounts(prevouts) {
    const stack = [];
    for (const { value } of prevouts) {
        stack.push(encodeValue(value));
    }
    return Buff$1.join(stack).digest;
}
function hashScripts(prevouts) {
    const stack = [];
    for (const { scriptPubKey } of prevouts) {
        stack.push(encodeScript(scriptPubKey, true));
    }
    return Buff$1.join(stack).digest;
}
function hashOutputs(vout) {
    const stack = [];
    for (const { value, scriptPubKey } of vout) {
        stack.push(encodeValue(value));
        stack.push(Script.encode(scriptPubKey, true));
    }
    return Buff$1.join(stack).digest;
}
function hashOutput(vout) {
    return Buff$1.join([
        encodeValue(vout.value),
        Script.encode(vout.scriptPubKey, true)
    ]).digest;
}
function getAnnexData(witness) {
    if (witness === undefined)
        return;
    if (witness.length < 2)
        return;
    let annex = witness.at(-1);
    if (typeof annex === 'string') {
        annex = Buff$1.hex(annex);
    }
    if (annex instanceof Uint8Array &&
        annex[0] === 0x50) {
        return Buff$1.raw(annex).prefixSize('be').digest;
    }
    return undefined;
}
function getPrevout(vin) {
    if (vin.prevout === undefined) {
        throw new Error('Prevout data missing for input: ' + String(vin.txid));
    }
    return vin.prevout;
}

const FIELD_SIZE = 0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2fn;
const CURVE_ORDER = 0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141n;
function signTx(seckey, txdata, index, config = {}) {
    const { sigflag = 0x00 } = config;
    const hash = hashTx(txdata, index, config);
    const sig = sign(seckey, hash);
    return (sigflag === 0x00)
        ? Buff$1.raw(sig)
        : Buff$1.join([sig, sigflag]);
}
function sign(secret, message, rand = Buff$1.random(32)) {
    const m = Buff$1.bytes(message);
    const dp = new Field(secret);
    const P = dp.point;
    const d = (P.hasEvenY) ? dp.big : dp.negated.big;
    const a = hashTag('BIP0340/aux', Buff$1.bytes(rand));
    const t = d ^ a.big;
    const n = hashTag('BIP0340/nonce', t, P.x.raw, m);
    const kp = new Field(n);
    const R = kp.point;
    const k = (R.hasEvenY) ? kp.big : kp.negated.big;
    const e = new Field(hashTag('BIP0340/challenge', R.x.raw, P.x.raw, m));
    const s = new Field(k + (e.big * d));
    return Buff$1.join([R.x.raw, s.raw]);
}
function verify(signature, message, pubkey, shouldThrow = false) {
    const P = new Point(xOnlyPub(pubkey));
    const m = Buff$1.bytes(message);
    const stream = Buff$1.bytes(signature).stream;
    if (stream.size < 64) {
        safeThrow('Signature length is too small: ' + String(stream.size), shouldThrow);
    }
    const r = stream.read(32);
    if (r.big > FIELD_SIZE) {
        safeThrow('Signature r value greater than field size!', shouldThrow);
    }
    const s = stream.read(32);
    if (s.big > CURVE_ORDER) {
        safeThrow('Signature s value greater than curve order!', shouldThrow);
    }
    const e = new Field(hashTag('BIP0340/challenge', r.raw, P.x.raw, m));
    const sG = new Field(s).point;
    const eP = P.mul(e.big);
    const R = sG.sub(eP);
    if (R.hasOddY) {
        safeThrow('Signature R value has odd Y coordinate!', shouldThrow);
    }
    if (R.x.big === 0n) {
        safeThrow('Signature R value is infinite!', shouldThrow);
    }
    return R.x.big === r.big;
}

const DEFAULT_VERSION$1 = 0xc0;
function getTapTag(tag) {
    const htag = Buff$1.str(tag).digest;
    return Buff$1.join([htag, htag]);
}
function getTapLeaf(data, version = DEFAULT_VERSION$1) {
    return Buff$1.join([
        getTapTag('TapLeaf'),
        getVersion(version),
        Buff$1.normalize(data)
    ]).digest.hex;
}
function getTapScript(script, version) {
    return getTapLeaf(Script.fmt.toBytes(script), version);
}
function getTapBranch(leafA, leafB) {
    if (leafB < leafA) {
        [leafA, leafB] = [leafB, leafA];
    }
    return Buff$1.join([
        getTapTag('TapBranch'),
        Buff$1.hex(leafA).raw,
        Buff$1.hex(leafB).raw
    ]).digest.hex;
}
function getTapRoot(leaves) {
    return Buff$1.hex(merkleize(leaves)[0]);
}
function merkleize(taptree, target, path = []) {
    const leaves = [];
    const tree = [];
    if (taptree.length < 1) {
        throw new Error('Tree is empty!');
    }
    for (let i = 0; i < taptree.length; i++) {
        const leaf = taptree[i];
        if (Array.isArray(leaf)) {
            const [r, t, p] = merkleize(leaf, target);
            target = t;
            leaves.push(r);
            for (const e of p) {
                path.push(e);
            }
        }
        else {
            leaves.push(leaf);
        }
    }
    if (leaves.length === 1) {
        return [leaves[0], target, path];
    }
    leaves.sort();
    if (leaves.length % 2 !== 0) {
        leaves.push(leaves[leaves.length - 1]);
    }
    for (let i = 0; i < leaves.length - 1; i += 2) {
        const branch = getTapBranch(leaves[i], leaves[i + 1]);
        tree.push(branch);
        if (typeof target === 'string') {
            if (target === leaves[i]) {
                path.push(leaves[i + 1]);
                target = branch;
            }
            else if (target === leaves[i + 1]) {
                path.push(leaves[i]);
                target = branch;
            }
        }
    }
    return merkleize(tree, target, path);
}
function getVersion(version = 0xc0) {
    return version & 0xfe;
}

function getTapTweak(key, data = new Uint8Array(), isPrivate = false) {
    const pub = (isPrivate)
        ? new Field(key).point.x.raw
        : xOnlyPub(key);
    return Buff$1.join([getTapTag('TapTweak'), pub, Buff$1.bytes(data)]).digest;
}
function getTweakedKey(intkey, data, isPrivate = false) {
    if (data === undefined)
        data = new Uint8Array();
    const k = Buff$1.bytes(intkey);
    const t = getTapTweak(intkey, data, isPrivate);
    if (isPrivate) {
        return tweakSecKey(k, t);
    }
    else {
        return tweakPubKey(k, t);
    }
}
function getTweakedPub(pubkey, data) {
    return getTweakedKey(pubkey, data);
}
function getTweakedSec(seckey, data) {
    return getTweakedKey(seckey, data, true);
}
function tweakSecKey(seckey, tweak) {
    let sec = new Field(seckey);
    if (sec.point.hasOddY) {
        sec = sec.negate();
    }
    return Buff$1.raw(sec.add(tweak).raw);
}
function tweakPubKey(pubkey, tweak) {
    pubkey = xOnlyPub(pubkey);
    const P = new Point(pubkey);
    const Q = P.add(tweak);
    return Buff$1.raw(Q.raw);
}

const DEFAULT_VERSION = 0xc0;
function getTapSecKey(seckey, config = {}) {
    return getTapKey(seckey, { ...config, isPrivate: true });
}
function getTapPubKey(pubkey, config = {}) {
    return getTapKey(pubkey, { ...config, isPrivate: false });
}
function getTapKey(intkey, config = {}) {
    const { isPrivate = false, tree = [], version = DEFAULT_VERSION } = config;
    const pubkey = (isPrivate)
        ? new SecretKey(intkey).pub.x.raw
        : xOnlyPub(intkey);
    let { target } = config;
    if (target !== undefined)
        target = Buff$1.bytes(target).hex;
    let tapkey, ctrlpath = [];
    if (tree.length > 0) {
        const [root, _t, path] = merkleize(tree, target);
        ctrlpath = path;
        tapkey = getTweakedKey(intkey, root, isPrivate);
    }
    else {
        if (target !== undefined) {
            tapkey = getTweakedKey(intkey, target, isPrivate);
        }
        else {
            tapkey = getTweakedKey(intkey, undefined, isPrivate);
        }
    }
    const parity = (isPrivate)
        ? new SecretKey(tapkey).point.raw[0]
        : tapkey[0];
    const cbit = Buff$1.num(version + readParityBit(parity));
    const block = [cbit, pubkey];
    if (ctrlpath.length > 0) {
        ctrlpath.forEach(e => block.push(Buff$1.hex(e)));
    }
    const cblock = Buff$1.join(block);
    if (target !== undefined) {
        if (!checkPath(tapkey, target, cblock, config)) {
            throw new Error('Path checking failed! Unable to generate path.');
        }
    }
    return [xOnlyPub(tapkey).hex, cblock.hex];
}
function checkPath(tapkey, target, cblock, config = {}) {
    const { isPrivate = false, throws = false } = config;
    const { parity, paths, intkey } = readCtrlBlock(cblock);
    const pub = (isPrivate)
        ? new SecretKey(tapkey).pub.x.raw
        : xOnlyPub(tapkey);
    const extkey = Buff$1.join([parity, pub]);
    if (extkey.length !== 33) {
        return safeThrow('Invalid tapkey: ' + extkey.hex, throws);
    }
    let branch = Buff$1.bytes(target).hex;
    for (const path of paths) {
        branch = getTapBranch(branch, path);
    }
    const k = getTweakedKey(intkey, branch);
    return (Buff$1.raw(k).hex === Buff$1.raw(extkey).hex);
}
function readCtrlBlock(cblock) {
    const buffer = new Stream$1(Buff$1.bytes(cblock));
    const cbyte = buffer.read(1).num;
    const intkey = buffer.read(32);
    const [version, parity] = (cbyte % 2 === 0)
        ? [cbyte, 0x02]
        : [cbyte - 1, 0x03];
    const paths = [];
    while (buffer.size >= 32) {
        paths.push(buffer.read(32).hex);
    }
    if (buffer.size !== 0) {
        throw new Error('Non-empty buffer on control block: ' + String(buffer));
    }
    return { intkey, paths, parity, version };
}
function readParityBit(parity = 0x02) {
    if (parity === 0 || parity === 1)
        return parity;
    if (parity === 0x02 || parity === '02')
        return 0;
    if (parity === 0x03 || parity === '03')
        return 1;
    throw new Error('Invalid parity bit: ' + String(parity));
}

async function verifyTx(txdata, index, config = {}) {
    const tx = Tx.fmt.toJson(txdata);
    const { throws = false } = config;
    const { prevout, witness = [] } = tx.vin[index];
    const witnessData = Tx.util.readWitness(witness);
    const { cblock, script, params } = witnessData;
    let pub;
    if (params.length < 1) {
        return safeThrow('Invalid witness data: ' + String(witness), throws);
    }
    const { scriptPubKey } = prevout ?? {};
    if (scriptPubKey === undefined) {
        return safeThrow('Prevout scriptPubKey is empty!', throws);
    }
    const { type, data: tapkey } = Tx.util.readScriptPubKey(scriptPubKey);
    if (type !== 'p2tr') {
        return safeThrow('Prevout script is not a valid taproot output:' + tapkey.hex, throws);
    }
    if (tapkey.length !== 32) {
        return safeThrow('Invalid tapkey length: ' + String(tapkey.length), throws);
    }
    if (cblock !== null &&
        script !== null) {
        const version = cblock[0] & 0xfe;
        const target = getTapLeaf(script, version);
        config.extension = target;
        if (!checkPath(tapkey, target, cblock, { throws })) {
            return safeThrow('cblock verification failed!', throws);
        }
    }
    if (config.pubkey !== undefined) {
        pub = Buff$1.bytes(config.pubkey);
    }
    else if (params.length > 1 && params[1].length === 32) {
        pub = Buff$1.bytes(params[1]);
    }
    else {
        pub = Buff$1.bytes(tapkey);
    }
    const rawsig = Script.fmt.toParam(params[0]);
    const stream = new Stream$1(rawsig);
    const signature = stream.read(64).raw;
    if (stream.size === 1) {
        config.sigflag = stream.read(1).num;
        if (config.sigflag === 0x00) {
            return safeThrow('0x00 is not a valid appended sigflag!', throws);
        }
    }
    const hash = hashTx(tx, index, config);
    if (!verify(signature, hash, pub, throws)) {
        return safeThrow('Invalid signature!', throws);
    }
    return true;
}

const TRSigner = {
    hash: hashTx,
    sign: signTx,
    verify: verifyTx
};

const Signer = {
    segwit: SWSigner,
    taproot: TRSigner
};

const TapTree = {
    getTag: getTapTag,
    getLeaf: getTapLeaf,
    getBranch: getTapBranch,
    getRoot: getTapRoot
};
const TapUtil = {
    readCtrlBlock: readCtrlBlock,
    readParityBit: readParityBit
};
const TapTweak = {
    getPubKey: getTweakedPub,
    getSecKey: getTweakedSec,
    getTweak: getTapTweak,
    tweakSecKey: tweakSecKey,
    tweakPubKey: tweakPubKey
};
const Tap = {
    getPubKey: getTapPubKey,
    getSecKey: getTapSecKey,
    encodeScript: getTapScript,
    checkPath: checkPath,
    tree: TapTree,
    tweak: TapTweak,
    util: TapUtil
};

class TxScript {
    constructor(script) {
        this._buff = Buff$1.raw(encodeScript(script));
    }
    get raw() {
        return this._buff.raw;
    }
    get hex() {
        return this._buff.hex;
    }
    get asm() {
        return decodeScript(this._buff);
    }
    getHash(format, version) {
        switch (format) {
            case 'p2w':
                return this._buff.toHash('hash256').hex;
            case 'p2sh':
                return this._buff.toHash('hash160').hex;
            case 'p2tr':
                return TapTree.getLeaf(this._buff, version);
            default:
                throw new Error('Unrecognized format: ' + format);
        }
    }
    toJSON() {
        return this.asm ?? [];
    }
}

const MAX_VAL = 0xFFFFFFFF;
const NO_LOCK = (1 << 31);
const TIME_MOD = 512;
const LOCK_TYPE = (1 << 22);
class TxSequence {
    constructor(value) {
        if (typeof value === 'string') {
            this.value = parseInt(value, 16);
        }
        else {
            this.value = value;
        }
    }
    get isReplaceable() {
        return this.value < MAX_VAL;
    }
    get isLocked() {
        return !(this.value !== MAX_VAL || (this.value & NO_LOCK) !== 0);
    }
    get isTimelock() {
        return (this.value & LOCK_TYPE) !== 0;
    }
    get timestamp() {
        return this.isLocked
            ? this.isTimelock
                ? this.value * TIME_MOD
                : this.value * TIME_MOD * 600
            : 0;
    }
    set timestamp(value) {
        this.value = Math.ceil(value / TIME_MOD);
    }
    get blockheight() {
        return this.isLocked
            ? !this.isTimelock
                ? this.value
                : Math.ceil((this.value * TIME_MOD) / 600)
            : 0;
    }
    set blockheight(value) {
        this.value = value;
    }
    get estDate() {
        return this.isTimelock
            ? new Date(Date.now() + (this.value * TIME_MOD * 1000))
            : new Date(Date.now() + (this.value * 600 * 1000));
    }
    set estDate(date) {
        const delta = date.getTime() - Date.now();
        this.value = (delta > (TIME_MOD * 1000))
            ? Math.ceil(delta / 1000 / TIME_MOD)
            : 1;
    }
    toJSON() {
        return this.value;
    }
}

class TxOutput {
    constructor(txout) {
        this.value = BigInt(txout.value);
        this.scriptPubKey = new TxScript(txout.scriptPubKey);
    }
}

class TxWitness {
    constructor(data, format) {
        this._data = data;
        this._meta = readWitness(data);
        this.format = format;
    }
    get length() {
        return this._data.length;
    }
    get annex() {
        const annex = this._meta.annex;
        return (annex !== null)
            ? Buff$1.raw(annex).hex
            : undefined;
    }
    get cblock() {
        const cblock = this._meta.cblock;
        return (cblock !== null)
            ? Buff$1.raw(cblock).hex
            : undefined;
    }
    get script() {
        const script = this._meta.script;
        return (script !== null)
            ? Script.decode(script)
            : undefined;
    }
    get params() {
        return this._meta.params;
    }
    toJSON() {
        return this._data;
    }
}

class TxInput {
    constructor(txin) {
        const { scriptSig = [], sequence = 0xfffffffd, witness = [] } = txin;
        this.txid = txin.txid;
        this.vout = txin.vout;
        this.scriptSig = new TxScript(scriptSig);
        this.sequence = new TxSequence(sequence);
        this.witness = new TxWitness(witness);
        if (txin.prevout !== undefined) {
            this.prevout = new TxOutput(txin.prevout);
        }
    }
}

const LOCKTIME_THRESHOLD = 500000000;
class TxLocktime {
    constructor(value = 0) {
        this.value = Buff$1.bytes(value).num;
    }
    get isTimelock() {
        return this.value > LOCKTIME_THRESHOLD;
    }
    get timestamp() {
        return this.isTimelock
            ? this.value
            : this.value * 600;
    }
    set timestamp(value) {
        this.value = value;
    }
    get blockheight() {
        return !this.isTimelock
            ? this.value
            : Math.floor(this.value / 600);
    }
    set blockheight(value) {
        this.value = value;
    }
    get estDate() {
        return this.isTimelock
            ? new Date(Date.now() + (this.value * 1000))
            : new Date(Date.now() + (this.value * 600 * 1000));
    }
    set estDate(date) {
        this.value = Math.floor((date.getTime() - Date.now()) / 1000);
    }
    toJSON() {
        return this.value;
    }
}

class Transaction {
    constructor(txdata) {
        if (typeof txdata === 'string') {
            txdata = Buff$1.hex(txdata).raw;
        }
        if (txdata instanceof Uint8Array) {
            txdata = decodeTx(txdata);
        }
        const data = txdata;
        this.version = data.version ?? 2;
        this.vin = data.vin?.map(e => new TxInput(e)) ?? [];
        this.vout = data.vout?.map(e => new TxOutput(e)) ?? [];
        this.locktime = new TxLocktime(data.locktime);
    }
    get data() {
        return JSON.parse(JSON.stringify(this));
    }
    get base() {
        return encodeTx(this.data, true);
    }
    get buff() {
        return encodeTx(this.data);
    }
    get raw() {
        return this.buff.raw;
    }
    get hex() {
        return this.buff.hex;
    }
    get size() {
        return this.raw.length;
    }
    get bsize() {
        return this.base.length;
    }
    get weight() {
        return this.bsize * 3 + this.size;
    }
    get vsize() {
        const remainder = (this.weight % 4 > 0) ? 1 : 0;
        return Math.floor(this.weight / 4) + remainder;
    }
    get hash() {
        return this.buff.toHash('hash160').reverse().hex;
    }
    get txid() {
        return this.base.toHash('hash256').reverse().hex;
    }
    async export() {
        const { size, weight, vsize, hex } = this;
        const txid = this.txid;
        const hash = this.hash;
        return { txid, hash, ...this.data, size, weight, vsize, hex };
    }
}

export { Address, TxInput as Input, TxOutput as Output, Script, TxSequence as Sequence, Signer, Tap, Transaction, Tx, TxWitness as Witness };
//# sourceMappingURL=module.js.map
